//***********************************************************************************************************************************************
//**                                                         RAZMASYNTH TELEWIZOR
//**                                                     (c) Benoit Bender & Kévin Tkac 2016
//**                                                    
//**
//***********************************************************************************************************************************************

/*INFO:


 -Ressource de la librairie: https://code.google.com/p/arduino-tvout/
 
 Cablage de l'arduino Uno:
 -Sortie Sync sur Digital 9 + resistance de 1K (Sur les petits écran de Telewizor c'est 750r)
 -Sortie Video sur Digital 7 + Resistance 470r (Chez moi c'est 330r) Les deux sorties sont sommés par ces resistances pour former le signal composite.
 -Sortie audio de la librairie TVout sur digital 11, pas utilisé.
 -Boutton de Scale sur port 2
 -Boutton de sample sur port 3
 */


//cablage du bouton poussoire
//...........................

// un resistor de 2.2k relie le +5V au pin D2 
//le commutateur momentanné est relié à une borne au pin D2
//l'autre borne du commutateur est relié à la masse
//un condo de 470n est branché en parallele du commutateur



//***********************************************************************************************************************************************
//**
//**                                                           BIBLIOTHEQUES
//**
//***********************************************************************************************************************************************

//attention les bibliothèque doivent être installées sur l'ordinateur hôte

//permet l'utilisation d'une télévision, écran LCD couleur.....
#include <TVout.h>
#include <fontALL.h>

//  permet la lecture des type de variable classique uint8_t et le reste
#include <stdint.h>

//le logo d'initialisation + les Bitmaps d'animations
#include "LOGORAZ.h"
#include "CPTBLOODBABY.h" //Display a Cpt Blood Baby
#include "VIPER.h" //Dipslay the Viper Spaceship
#include "VIPERBLACK.h" //Display a black square around the Viper
#include "MAZINGER.h" //Display a Mazinger Z mecha


//***********************************************************************************************************************************************
//**
//**                                                     VARIABLES GLOBALES
//**
//***********************************************************************************************************************************************

//input pins     
const uint8_t inputPin = 0; // Le numero de la pin analo d'entrée CV
const uint8_t scalePin = 2; // Le numero de la pin du boutton de scale utilisé aussi pour faire défiler le menu textuel
const uint8_t samplePin = 3; // Le numero de la pin du boutton de sample urilisé aussi pour valider le choix du sous programme dans le menu textuel
const uint8_t menuPin = 4; //le numéro du poin pour la selection dans le menu

//pour les boutons poussoirs
uint8_t variable_couleur = 0 ;     // ne sert à rien pour l'instant mais tu pourras alimenter un switchcase avec (genre choix de menu)
uint8_t memoireBoutonCouleur=0;
uint8_t fm_BoutonCouleur=0;

//Pour l'oscilloscope                                         
const int graphOffsetX = 16;      //notes: avant 10
const int graphOffsetY = 94;     //notes: avant 94
const int graphWidth = 88;      //notes: avant 99
const int graphHeight = 86;     //notes: avant 86

const int upperGraphLimit = graphOffsetY + 1;
const int lowerGraphLimit = graphOffsetY - graphHeight;

int scaleFactor = 1024/graphHeight+1;
int offset = 0;
int sampleSize = -4;
// if sampleSize <= 0, it denotes the sampling speed.

int minValue, maxValue, avgValue;
long startTime = 0, sampleTime = 0;

//used if sampleSize <=3 to draw faster
int data[graphWidth];
uint8_t dispY[graphWidth];

// pour la gestion de la bibliothèque TVout
TVout TV;

//pour l'appel une fois du dessin du rectangle+echelle
uint8_t premiereFoisRectangle ;

//pour l'appel une fois du clear screnn
uint8_t premiereFoisClearScreen;
uint8_t premiereFoisClearScreen_2;
uint8_t premiereFoisClearScreen_3;
uint8_t premiereFoisClearScreen_4;
uint8_t premiereFoisClearScreen_5;
uint8_t premiereFoisClearScreen_6;
uint8_t premiereFoisClearScreen_7;
uint8_t premiereFoisClearScreen_8;

//pour le timer 0
//on pourra peut etre passer en uint8_t
//compteur principal ct comme compteur timer
uint16_t ct0 ;
uint16_t ct1 ;
uint16_t ct2 ;
uint16_t ct3 ;
uint16_t ct4 ;
uint16_t ct5 ;
uint16_t ct6 ;
uint16_t ct7 ;

//compteur relair cr
uint16_t cr0 ;
uint16_t cr1 ;
uint16_t cr2 ;
uint16_t cr3 ;
uint16_t cr4 ;
uint16_t cr5 ;
uint16_t cr6 ;
uint16_t cr7 ;

//commutateur des compteur relais
//intervient en ISR et en loop
uint8_t commutateurCompteur=0;

//permet une gestion du defilement des lignes
uint8_t ecart;

//compteur en timer0 à la fréquence réelle
uint16_t compteurISRglobal ;




//***********************************************************************************************************************************************
//**
//**                                                    INITIALISATION
//**
//***********************************************************************************************************************************************

void setup()
{                                         
  //    TIMER 0  61Hz
  //......................

  cli();//stop interrupts

  TCCR0A = 0;// set entire TCCR2A register to 0
  TCCR0B = 0;// same for TCCR2B
  TCNT0  = 0;//initialize counter value to 0
  // set compare match register for 2khz increments
  OCR0A = 255;// = (must be <256)
  // turn on CTC mode
  TCCR0A |= (1 << WGM01);
  // Set CS10 and CS12 bits for 1024 prescaler
  TCCR0B |= (1 << CS12) | (1 << CS10);  
  // enable timer compare interrupt
  TIMSK0 |= (1 << OCIE0A);

  //f=arduino clock sppeed/ ( prescaler * (compar match register +1) ) = 61.0351Hz


  //port série ATTENTION le port serie met du desordre
  //Serial.begin (9600) ;

  //pour les boutons poussoirs 
  pinMode(scalePin, INPUT);               // Initialize la pin du boutton comme une entree
  pinMode(samplePin, INPUT);              // Initialize la pin du boutton comme une entree
  pinMode(menuPin, INPUT);                // Initialize la pin du boutton comme une entree


  //initialisation de la bibliothèque TV: NTSC ou PAL
  //  TV.begin(PAL,120,96);                                  //reso 125*100, 120*98, 120*97, 120*99, 120*100, 120*101 ne fonctionnent pas sur ma télé   
  //reso 110*88, 120*96, 115*92          fonctionnent    
  TV.begin(NTSC,120,96);                                     //°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°Reso changée NTSC 120,96

    //pour les polices de caracteres
  TV.select_font(font4x6);

intro(); //Appelle de la fonction ""intro"

  //pour l'oscilloscope
  //dessine un rectangle sur l'écran (fonction du cleargraph)
  //avec l'échelle (fonction drawscale)
  clearGraph();
  drawScale(); 

  //permet de démarrer ou on veut dans le switch case
  //4 voltmetre
  //3 perspective 
  //2 invader
  //1 rubiks
  //0 oscillo
  variable_couleur=6;

  //juste pour le développement
 premiereFoisClearScreen = 1;
  premiereFoisClearScreen_2 = 1;
  premiereFoisClearScreen_3 = 1;
  premiereFoisClearScreen_4 = 1;
  premiereFoisClearScreen_5 = 1;
  premiereFoisClearScreen_6 = 1;
  premiereFoisClearScreen_7 = 1;
  premiereFoisClearScreen_8 = 1;

  //mise ne route TIMER 0
  sei();

}


//***********************************************************************************************************************************************
//**
//**                                                    INITIALISATION DE L OSCILLOSCOPE
//**
//**                                 **********     OSCILLOSCOPE       fonction secondaire      **********
//**
//**
//***********************************************************************************************************************************************



//..............................................................................
//                                  initADC
void initADC(int speed)
{
  // speed == 1: normal analogread.
  // speed in [-4,0]: faster (-4 is the fastest)

  ADCSRA = 0x87; //default
  if ( speed >= 1 ) ADCSRA = 0x87;   // turn on adc, freq  = 1/128,  125 kHz. 
  if ( speed == 0 ) ADCSRA = 0x86;   // turn on adc, freq  = 1/64,   250 kHz. 
  if ( speed == -1 ) ADCSRA = 0x85;   // turn on adc, freq  = 1/32,   500 kHz. 
  if ( speed == -2 ) ADCSRA = 0x84;   // turn on adc, freq  = 1/16 ,    1 MHz. 
  if ( speed == -3 ) ADCSRA = 0x83;   // turn on adc, freq  = 1/8,      2 MHz. 
  if ( speed == -4 ) ADCSRA = 0x82;   // turn on adc, freq  = 1/4,      4 MHz. 
  // does not seem to work:
  // if ( speed == -5 ) ADCSRA = 0x81;   // turn on adc, freq  = 1/2,      8 MHz.  

  ADMUX = 0x40;
  ADCSRA |= (1<<ADSC);
  while(!(ADCSRA & 0x10));
}



//..............................................................................
//                                    resetADC
void resetADC()
{
  ADCSRA = 0x00;
}



//..............................................................................
//                                 readFast
void readFast(int speed)
{
  // fills array with data on analog-pin 0.
  // speed >= 1: sample "speed" times and take the adverage.
  // speed in [-4,0]: read faster (-4 is the fastest)

  int * data_ptr = data; // pointer to array

  volatile char i = graphWidth; // iterator in array, init with array size
  volatile int val = 0, sum;

  initADC(speed);

  startTime = micros(); // set startTime here, to get a more accurate rate
  if(speed > 1){
    ADCSRA |= (1<<ADSC); // start reading the first value
    do{
      sum = 0;
      for(int j=0; j<speed; j++){
        while(!(ADCSRA & 0x10)); // wait for output
        val = ADCL;  // read out low data bits
        val += (ADCH << 8); // read out high data bits
        ADCSRA |= (1<<ADSC); // already start reading the next value for more speed
        sum += val;
      }
      *data_ptr++ = sum / speed;
    }
    while(--i);
  }
  else{
    ADCSRA |= (1<<ADSC); // start reading the first value
    do{
      while(!(ADCSRA & 0x10));
      val = ADCL;
      val += (ADCH << 8);
      ADCSRA |= (1<<ADSC); // already start reading the next value for more speed
      *data_ptr++ = val;
    }
    while(--i);
  }
  resetADC();
}



//..............................................................................
//                                       waitFor
void waitFor(int speed, int threshold)
{
  volatile int val = 0;

  initADC(speed);

  boolean wasLow = false;
  ADCSRA |= (1<<ADSC); // start reading first value
  for(int w=0 ; w<400 ; w++){
    while(!(ADCSRA & 0x10));
    val = ADCL;
    val += (ADCH << 8);
    ADCSRA |= (1<<ADSC); // already start reading next value for more speed
    if(val < threshold){
      wasLow = true;
    }
    else if((val > threshold) && wasLow){
      break;
    }
  }
  resetADC();
}



//..............................................................................
//                                       drawPixel
void drawPixel(int x, int y, char c)
{
  if((y > lowerGraphLimit) && (y < upperGraphLimit)){
    TV.set_pixel(x,y,c);
  }
}

int val2y(int val){
  return graphOffsetY - val/scaleFactor - offset;
}



//..............................................................................
//                      drawScale (met les échelles sur le rectangle, modifié pour affichage d'un signal bipolaire +/-5V
void drawScale()
{
  //delete old scale
  TV.draw_rect(0,lowerGraphLimit, graphOffsetX-2,graphHeight+1,0,0);

  for(int i = 0; i<=40;i++){
    int y = val2y(1024 * i / 20);   // i/20 soit 20 segments
    drawPixel(graphOffsetX-2,y,1); // Affiche les 20 graduation primaires de l'echelle Px par Px
    drawPixel(graphOffsetX-3,y,1);

    // Grisé car utile en scale fois xxx
    //   if((i % 2 != 0) && (scaleFactor < 8) && (y > lowerGraphLimit + 3) && (y < upperGraphLimit - 3)){
    //    TV.print(graphOffsetX - 18,y-3,i/4.0,2);
    // } 


    if(i % 2 == 0){
      drawPixel(graphOffsetX-4,y,1);  


      // Grisé car utile en scale fois xxx
      // if((i % 4 != 0) && (scaleFactor <=8) && (y > lowerGraphLimit + 3) && (y < upperGraphLimit - 3)){
      //  TV.print(graphOffsetX - 15,y-3,i/4.0,1);
      // } 

    }


    if(i % 10 == 0){
      drawPixel(graphOffsetX-5,y,1);
      drawPixel(graphOffsetX-6,y,1);

      if((y > lowerGraphLimit + 3) && (y < upperGraphLimit - 3)){
        TV.print(graphOffsetX - 12,y-5,"0");    // Position du zéro sur l'echelle, "0" ou i -i
      }
    } 
  }
}



//..............................................................................
//                        clearGraph  (dessine un rectangle)  
void clearGraph()
{
  TV.draw_rect(graphOffsetX - 1, graphOffsetY - graphHeight, graphWidth+1, graphHeight+1, 1, 0);

}



//..............................................................................
//                               drawData
void drawData(int i, int val)
{
  // we use dispY[i] to store the displayed pixel. so we only need to
  // erase that pixel, and not the whole column.
  int y = val2y(val);
  int x = i + graphOffsetX;
  drawPixel(x,dispY[i],0);
  drawPixel(x,y,1);
  dispY[i] = y;
}



//..............................................................................
//                              drawData
void drawData(int i, int v1, int v2)
{
  int u = val2y(v1);
  int v = val2y(v2);
  int x = i + graphOffsetX;
  TV.draw_column(x, graphOffsetY, graphOffsetY - graphHeight + 1, 0);
  u = min(max(u,lowerGraphLimit),upperGraphLimit);
  v = min(max(v,lowerGraphLimit),upperGraphLimit);
  TV.draw_column(x, u, v, 1);
}



//..............................................................................
//                           changeScalefactor
void changeScaleFactor()
{
  int i = 1;
  while(i < scaleFactor)
    i = i*2;
  scaleFactor = i / 2;
  if(scaleFactor == 0){
    scaleFactor = 1024/graphHeight+1;
  }
  offset = graphHeight/2 - avgValue/scaleFactor;

  // we need: 0/scaleFactor + offset >= 0 and 1024/scaleFactor + offset <= graphHeight
  offset = max(graphHeight - 1024/scaleFactor,offset);
  offset = min(0,offset);

  drawScale();
  clearGraph();
}


//..............................................................................
//                              changeSampleSize
void changeSampleSize()
{
  if(sampleSize<5)
    sampleSize = sampleSize + 1;
  else
    //sampleSize = sampleSize * 2;
    sampleSize = sampleSize * 8 / 5;
  if(sampleSize > 300){
    sampleSize = -4;
  }
  sampleTime = 0;
  clearGraph();
}



//..............................................................................
//                           PAS DANS UNE FONCTION
//                   lecture de bouton poussoir archaïque
boolean scaleButtonPressed = false, sampleButtonPressed = false;
boolean checkButtons()
{
  if(digitalRead(scalePin) == HIGH){
    scaleButtonPressed = true;
  }
  else{
    if(scaleButtonPressed){
      changeScaleFactor();
      scaleButtonPressed = false;
      return true;
    }
  }
  if(digitalRead(samplePin) == HIGH){
    sampleButtonPressed = true;
  }
  else{
    if(sampleButtonPressed){
      changeSampleSize();
      sampleButtonPressed = false;
      return true;
    }
  }
  return false;
}

float val2volts(int val){
  return val * 5 / 1024.0;
}




//............................................................................................. printVolts
//                                                                                        
void printVolts(){

  TV.print(5,0,"V: "); 
  TV.print((val2volts (avgValue*2) -5),2); // Affiche la tension Bipolaire ou Unipolaire réelle ( Applique une correction au CV analo /2 +5V offset induit par le hardware)


  TV.print(45,0,"VPP: "); 
  TV.print(val2volts(avgValue *2),2); // Affiche l'amplitude du signal entre 0 et 10V (Volts Peak to Peak)



}


/*
//............................................................................................. printVolts_2 pour VOLTMETER
//                                                                                        
void printVolts_2(){

  TV.print(40,50,"V: "); 
  TV.print((val2volts (avgValue*2) -5),2); // Affiche la tension Bipolaire ou Unipolaire réelle ( Applique une correction au CV analo /2 +5V offset induit par le hardware)




}
*/


void printSampleRate(){
  long x = graphWidth;
  x = x*1000000/sampleTime;
  if(x > 9999){
    TV.print(90,0, x/1000);
    TV.print("kHz  ");
  }
  else{
    TV.print(90,0, x);
    TV.print("Hz ");
  }
  //TV.print(sampleSize);
  //TV.print("  ");
}






//***********************************************************************************************************************************************
//**
//**                                                    TIMER 0
//**
//***********************************************************************************************************************************************


ISR(TIMER0_COMPA_vect)
{

  //    rappel variables globales
  //.................................
  //détermine l'écartement entre les différente ligne 
  //max = 10
  //mini = 7
  //c'est un uint8_t    "ecart"
  // en uint16 compteurISRglobal


  //  variables locales
  //..........................
  static uint8_t ISRcompteur ;
  ISRcompteur ++;
  const uint8_t compareMatchRegister = 2; 



  //pour une autres fonction d'affichage
  //.......................................
  compteurISRglobal ++ ;


  //   DIVISION DE FREQUENCE  
  //.................................
  //ici on virtualise le timer en implémentant un timer dans le timer.... :)

  if (ISRcompteur >  compareMatchRegister )
  {



    //A partir de là tout est virtualisé !!    :)
    //................................................................



    // differents compteur sur timer 0
    //..................................  

    //ct0 est le compteur "maitre" c'est de lui que viennent les autres compteurs
    ct0 ++ ;



    //          explications
    //........................................

    //il y a 2 groupes de compteurs, u ngroupe principal et un groupe de relais
    //le groupe principal est composé de 8 compteurs ct0 à ct7 (ct comme compteur timer)
    //ceux ci démarrre à interval régulier "ecart"
    //ces  compteurs sont remis à "0" quand le premier atteint la reinitialisation du Compteur "reInitialisation"

    //le groupe de relai est composé de compteurs; il servent à combler les "trous" laissés par la remise à zero des compteur du groupe principal
    //ils commence quand le "commutateurCompteur" passe à "1"
    //ils sont reinitialisé quand le dernier compteur atteint le "reInitialisation"  
    //ils sont synchronisés au groupe principal quand démarre le un compteur principal  
    //ils sont nommés cr (cr comme compteur relais) 




    //  l'initialisation des compteur principaux et compteur relais pilote
    //......................................................................
    if (ct0 > ecart*1)
    {
      ct1 ++;

    }
    if (ct0 > ecart*2)
    {
      ct2 ++;

    }
    if (ct0 > ecart*3)
    {
      ct3 ++;  

    }
    if (ct0 > ecart*3)
    {
      ct4 ++;  

    }
    if (ct0 > ecart*4)
    {
      ct5 ++; 
      commutateurCompteur=1;  
    }
    if (ct0 > ecart*5)
    {
      ct6 ++; 

    }
    if (ct0 > ecart*6)
    {
      ct7 ++; 

    } 



    //  l'initialisation des compteur  relais pilote
    //....................................................
    if (1 == commutateurCompteur)
    {
      cr0 ++;
    }
    if (cr0 > ecart*1)
    {
      cr1 ++;
    }
    if (cr0 > ecart*2)
    {
      cr2 ++;
    }
    if (cr0 > ecart*3)
    {
      cr3 ++;
    }
    if (cr0 > ecart*4)
    {
      cr4 ++;
    }
    if (cr0 > ecart*5)
    {
      cr5 ++;
    }





    // ré initialisation des variables
    //...................................

    //il faut reajuster le delai minimal lorsque l'on touche à l'"ecart" entre les lignes 
    //c'est la raison pour laquelle on multiple ecart par 4 avec "ecart<<2"
    //on ne peut dépasser une valeur maximal sinon les ligne viennent de tout en haut de l'écran
    //par calcul on obtient le mini  ctx*ctx/32=40 quand y=90


    const uint8_t reInitialisation =  ( ( ecart << 2 ) + 44 );

    if (ct0 >=   reInitialisation  )
    {
      ct0=0;  
      ct1=0;
      ct2=0;
      ct3=0;  
      ct4=0;
      ct5=0;
    }

    if ( ( cr0 >= reInitialisation )   )
    {
      cr0=0; 
      cr1=0;
      cr2=0;
      cr3=0;
      cr4=0;
      cr5=0;
      commutateurCompteur=0; 

    }


    //remise à zero du compteur 60Hz
    ISRcompteur = 0;

  }




}










//***********************************************************************************************************************************************
//**
//**                                                    PROGRAMME PRINCIPAL
//**
//***********************************************************************************************************************************************

void loop ( )
{

  //lecture du bouton poussoir de selection de programme 
  //renvoit une valeur "variable_couleur" de 0 à ? pour le switch case 
  // +++Pensez à augmenter le nombre dans "limitation de variable_mult" du code de commutateur, si ajout de switch case!!!
  
  BoutonPoussoir() ; 

  //explication sur les variables "premiereFoisclearScreen" et "premiereFoisRectangle"
  //ces variables permettent de faire tourner une fois et une seul une petit boucle contenue dans une conditionnelle if(bla bla)
  //par exemple pour dessiner le rectangle et l'échelle de l'oscillo ou pour faire un clear screen




  //                  SELECTION AFFICHAGE
  //***************************************************************                               SWITCH/CASE

  switch(variable_couleur)
    //switch(inputPinState)
    
   {
  
  case 0:
    //VIPER
    if (1 == premiereFoisClearScreen )
    {
      TV.clear_screen();
      premiereFoisClearScreen = 0;
    }

    //VIPER MARK II
    vipersMarkII();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    //ré initialisation de la variable
    premiereFoisRectangle=1;
    premiereFoisClearScreen_2 = 1;
    premiereFoisClearScreen_3 = 1;
    premiereFoisClearScreen_4 = 1;
    premiereFoisClearScreen_5 = 1;
    premiereFoisClearScreen_6 = 1;
    premiereFoisClearScreen_7 = 1;
    premiereFoisClearScreen_8 = 1;
    break;


  case 1:
    //OSCILLOSCOPE
    //rectangle et echelle
    if (1 == premiereFoisRectangle)
    {   
         TV.clear_screen();
      clearGraph();
      drawScale(); 
      TV.select_font(font4x6); 
      //reinitialisation de la variable
      premiereFoisRectangle=0;
    }
    //oscilloscope
    oscilloscope();

     
    //ré initialisation de l a variable
    premiereFoisClearScreen = 1;
    premiereFoisClearScreen_2 = 1;
    premiereFoisClearScreen_3 = 1;
    premiereFoisClearScreen_4 = 1;
    premiereFoisClearScreen_5 = 1;
    premiereFoisClearScreen_6 = 1;
    premiereFoisClearScreen_7 = 1;
    premiereFoisClearScreen_8 = 1;
    break;

  case 2:
    //RUBIKS
    if (1 == premiereFoisClearScreen_2 )
    {
      TV.clear_screen();
      //ré initialisation de la variable
      premiereFoisClearScreen_2 = 0;
    }

    rubiks();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus
    
    //ré initalisation de la variable
    premiereFoisClearScreen = 1;
    premiereFoisClearScreen_3 = 1;
    premiereFoisClearScreen_4 = 1;
    premiereFoisClearScreen_5 = 1;
    premiereFoisClearScreen_6 = 1;
    premiereFoisClearScreen_7 = 1;
    premiereFoisClearScreen_8 = 1;
    premiereFoisRectangle=1;
    break;

  case 3:
    //INVADER
    if (1 == premiereFoisClearScreen_3 )
    {
      TV.clear_screen();
      //ré initialisation de la variable
      premiereFoisClearScreen_3 = 0;
    }

    invader();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus
    
    //ré initalisation de la variable
    premiereFoisClearScreen = 1;
    premiereFoisClearScreen_2 = 1;
    premiereFoisClearScreen_4 = 1;
    premiereFoisClearScreen_5 = 1;
    premiereFoisClearScreen_6 = 1;
    premiereFoisClearScreen_7 = 1;
    premiereFoisClearScreen_8 = 1;
    premiereFoisRectangle=1;
    break;

  case 4:
    //PERSPECTIVE
    if (1 == premiereFoisClearScreen_4 )
    {
      TV.clear_screen();
      //ré initialisation de la variable
      premiereFoisClearScreen_4 = 0;
    }

    perspective();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus
    
    //ré initalisation de la variable
    premiereFoisClearScreen = 1;
    premiereFoisClearScreen_2 = 1;
    premiereFoisClearScreen_3 = 1;
    premiereFoisClearScreen_5 = 1;
    premiereFoisClearScreen_6 = 1;
    premiereFoisClearScreen_7 = 1;
    premiereFoisClearScreen_8 = 1;
    premiereFoisRectangle=1;
    break;

  case 5:
    //WARP
    if (1 == premiereFoisClearScreen_5 )
    {
      TV.clear_screen();
      //ré initialisation de la variable
      premiereFoisClearScreen_5 = 0;
    }

    warp();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus
    
    //ré initalisation de la variable
    premiereFoisClearScreen = 1;
    premiereFoisClearScreen_2 = 1;
    premiereFoisClearScreen_3 = 1;
    premiereFoisClearScreen_4 = 1;
    premiereFoisClearScreen_6 = 1;
    premiereFoisClearScreen_7 = 1;
    premiereFoisClearScreen_8 = 1;
    premiereFoisRectangle=1;
    break;

  case 6:
    //SNAKE
    if (1 == premiereFoisClearScreen_6 )
    {
      TV.clear_screen();
      //ré initialisation de la variable
      premiereFoisClearScreen_6 = 0;
    }

    snake();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    //ré initalisation de la variable
    premiereFoisClearScreen = 1;
    premiereFoisClearScreen_2 = 1;
    premiereFoisClearScreen_3 = 1;
    premiereFoisClearScreen_4 = 1;
    premiereFoisClearScreen_5 = 1;
    premiereFoisClearScreen_7 = 1;
    premiereFoisClearScreen_8 = 1;
    premiereFoisRectangle=1;
    break;

case 7:
    //Cpt Blood
    if (1 == premiereFoisClearScreen_7 )
    {
      TV.clear_screen();
      //ré initialisation de la variable
      premiereFoisClearScreen_7 = 0;
    }

    CptBlood();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus
    
    //ré initalisation de la variable
    premiereFoisClearScreen = 1;
    premiereFoisClearScreen_2 = 1;
    premiereFoisClearScreen_3 = 1;
    premiereFoisClearScreen_4 = 1;
    premiereFoisClearScreen_5 = 1;
    premiereFoisClearScreen_6 = 1;
    //premiereFoisClearScreen_7 = 1;
    premiereFoisClearScreen_8 = 1;
    premiereFoisRectangle=1;
    break;


case 8:
    //Mazinger Z
    if (1 == premiereFoisClearScreen_8 )
    {
      TV.clear_screen();
      //ré initialisation de la variable
      premiereFoisClearScreen_8 = 0;
    }

    Mazinger();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus
    
    //ré initalisation de la variable
    premiereFoisClearScreen = 1;
    premiereFoisClearScreen_2 = 1;
    premiereFoisClearScreen_3 = 1;
    premiereFoisClearScreen_4 = 1;
    premiereFoisClearScreen_5 = 1;
    premiereFoisClearScreen_6 = 1;
    premiereFoisClearScreen_7 = 1;
    //premiereFoisClearScreen_8 = 1;
    premiereFoisRectangle=1;
    break;

  default:
    // Pas normal, on repart au début
    variable_couleur = 0;
  }

}



//***********************************************************************************************************************************************
//**
//**                                                    LOGICIELS SECONDAIRES
//**
//***********************************************************************************************************************************************



//                                                                SNAKE
//.........................................................................................................................................
//******************************************************************************************************************************************

void snake()
{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t yMilieu = vRes >> 2 ;
  const uint8_t blanc = 1;
  const uint8_t noir =0;
  const uint8_t radius = 2;
  const uint8_t tempsDaffichage = 1;
  uint8_t x;
  const uint8_t y = 1;



  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 3 )  );
  if (Value >= ( hRes -5) )
  {
    Value = ( hRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 




  //          pattern
  //............................

  //draw_circle(x,y,radius,color,fillcolor)
  TV.draw_circle(Value,yMilieu,radius,blanc,blanc);
  

  
  //set_pixel(x,y,color)
  x= random ( hRes) ;
  TV.set_pixel (x, y, blanc);
  TV.set_pixel (x, y+1, blanc);





  //mouvement de l'écran
  //shift(distance,direction)   direction =>  0=up 1 = down 2 = left 3=right 
  TV.shift(1,1) ; 




}


//                                                                 WARP
//.........................................................................................................................................
//******************************************************************************************************************************************
void warp()
{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;
  const uint8_t yMilieu = vRes >> 1 ;
  const uint8_t blanc = 1;
  const uint8_t noir =0;
  uint8_t x;
  uint8_t y;
  uint8_t tempsDaffichage;
  static uint8_t premiereFois = 1;



  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 16
  uint8_t Value= ( (  (int)avgValue >> 6 )  );
  tempsDaffichage = Value;
  if (tempsDaffichage < 5)
  {
    tempsDaffichage = 2;
  }


  //          pattern
  //............................
  x= random ( hRes) ;
  y= random ( vRes );

/*

  if (1 == premiereFois )
  {
    //draw_rect(x0,y0,width,height,color,fillcolor)
    //draw_circle(x,y,radius,color,fillcolor) 
    //ATTENTION a ne pas dépasser un radius de plus de 50
    TV.clear_screen();
    for (uint16_t i = 0 ; i<16317; i++)
    {
      x= random ( hRes) ;
      y= random ( vRes );
      TV.set_pixel (x, y, blanc);
    }
    TV.draw_rect(0,0,hRes,vRes,blanc,blanc);
    for (uint8_t i = 0 ; i<200; i++)
    {
      TV.draw_circle(xMilieu,yMilieu,i>>2,noir,noir);

    }
    
    
    //ré initialisation de la variable
    premiereFois = 0;
  }


*/



  if (compteurISRglobal<tempsDaffichage)
  {   
    TV.draw_line(xMilieu,yMilieu,x,y,2);
    
    

    //TV.draw_line(xMilieu,yMilieu,x,y,noir);

  }
  else
  {
    //desaffichage
    //draw_circle(x,y,radius,color,fillcolor)    ATTENTION a ne pas dépasser un radius de plus de 50

    for (uint8_t i = 0 ; i<14; i++)
    {
     TV.draw_circle(xMilieu,yMilieu,i<<2,0,0);

    }
  }



  //reinitialisation de la variable
  //................................
  //quand on atteint deux fois le temps d'affichage on reinitialise
  if (compteurISRglobal >=  (tempsDaffichage<<1) )
  {
    compteurISRglobal = 0;
  }





}



//                                                                       RUBIKS
//.........................................................................................................................................
//******************************************************************************************************************************************
void rubiks()        //Affiche des lignes selon ADC: Excellent visuelement avec LFO Shibuya ou petit signaux 0/5V de clock car fait semblant de réagir en fonctions de la vitesse
{

  //rappel variable global
  //.............................
  //compteurISRglobal compte en uint16 à 60 Hz


  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  uint8_t x  ;
  uint8_t y ;
  const uint8_t blanc = 1;
  const uint8_t noir =0;
  static uint16_t tempsDaffichage ;

  //     affichage
  //...................
  x= random ( hRes) ;
  y= random ( vRes );


  if (compteurISRglobal<tempsDaffichage)
  {


    TV.draw_line(x,0,x,96,blanc);
    TV.draw_line(0,y,120,y,blanc);


  }
  else
  {
    //desaffichage
    TV.draw_line(x,0,x,96,noir);
    TV.draw_line(0,y,120,y,noir);
  }

  //reinitialisation de la variable
  //................................

  if (compteurISRglobal >=  (tempsDaffichage<1) )
  {
    compteurISRglobal = 0;
  }




  //                   lecture ADC
  //........................................................

  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //tempsDaffichage valeur standard = 200

  int Value= (int)avgValue;
  tempsDaffichage=  Value>>8 ;


}


//                                                         INVADER SPRITE
//..........................................................................................................................................
//******************************************************************************************************************************************
void invader()
{

  //rappel variable global
  //.............................
  //compteurISRglobal compte en uint16 à 60 Hz


  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  uint8_t x  ;
  uint8_t y ;
  const uint8_t blanc = 1;
  const uint8_t noir =0;
  static uint16_t tempsDaffichage ;


  //     affichage
  //...................
  x= random ( hRes) ;
  y= random ( vRes );


  if (compteurISRglobal< (tempsDaffichage >> 3) )
  {
    //affichage invader sprite
    TV.set_pixel(x,y,blanc); 
    TV.set_pixel(x+1,y,blanc); 
    TV.set_pixel(x+2,y,blanc); 
    TV.set_pixel(x+3,y,blanc); 
    TV.set_pixel(x+4,y,blanc); 
    TV.set_pixel(x+5,y,blanc); 
    TV.set_pixel(x-1,y,blanc); 
    TV.set_pixel(x-2,y,blanc); 
    TV.set_pixel(x-3,y,blanc); 
    TV.set_pixel(x-4,y,blanc); 
    TV.set_pixel(x-5,y,blanc); 
    TV.set_pixel(x,y-1,blanc); 
    TV.set_pixel(x+1,y-1,blanc); 
    TV.set_pixel(x+3,y-1,blanc); 
    TV.set_pixel(x+4,y-1,blanc); 
    TV.set_pixel(x-1,y-1,blanc); 
    TV.set_pixel(x-3,y-1,blanc); 
    TV.set_pixel(x-4,y-1,blanc); 
    TV.set_pixel(x,y-2,blanc); 
    TV.set_pixel(x+1,y-2,blanc);
    TV.set_pixel(x+2,y-2,blanc);
    TV.set_pixel(x+3,y-2,blanc);
    TV.set_pixel(x-1,y-2,blanc);
    TV.set_pixel(x-2,y-2,blanc);
    TV.set_pixel(x-3,y-2,blanc);
    TV.set_pixel(x+2,y-3,blanc);
    TV.set_pixel(x-2,y-3,blanc);
    TV.set_pixel(x+3,y-4,blanc);
    TV.set_pixel(x-3,y-4,blanc);
    TV.set_pixel(x,y+1,blanc);
    TV.set_pixel(x+1,y+1,blanc);
    TV.set_pixel(x+2,y+1,blanc);
    TV.set_pixel(x+3,y+1,blanc);
    TV.set_pixel(x+5,y+1,blanc);
    TV.set_pixel(x-1,y+1,blanc);
    TV.set_pixel(x-2,y+1,blanc);
    TV.set_pixel(x-3,y+1,blanc);
    TV.set_pixel(x-5,y+1,blanc);
    TV.set_pixel(x+3,y+2,blanc);
    TV.set_pixel(x+5,y+2,blanc);
    TV.set_pixel(x-3,y+2,blanc);
    TV.set_pixel(x-5,y+2,blanc);
    TV.set_pixel(x+1,y+3,blanc);
    TV.set_pixel(x+2,y+3,blanc);
    TV.set_pixel(x-1,y+3,blanc);
    TV.set_pixel(x-2,y+3,blanc);

  }
  else
  {
    //desaffichage
    TV.set_pixel(x,y,noir);
    TV.set_pixel(x+1,y,noir);
    TV.set_pixel(x+3,y,noir);
    TV.set_pixel(x+4,y,noir);
    TV.set_pixel(x-2,y,noir);
    TV.set_pixel(x-3,y,noir);
    TV.set_pixel(x,y-1,noir);
    TV.set_pixel(x+1,y-1,noir);
    TV.set_pixel(x+2,y-1,noir);
    TV.set_pixel(x+3,y-1,noir);
    TV.set_pixel(x-1,y-1,noir);
    TV.set_pixel(x-2,y-1,noir);
    TV.set_pixel(x,y-2,noir);
    TV.set_pixel(x+1,y-2,noir);
    TV.set_pixel(x+2,y-2,noir);
    TV.set_pixel(x-1,y-2,noir);
    TV.set_pixel(x,y-3,noir);
    TV.set_pixel(x+1,y-3,noir);
    TV.set_pixel(x,y+1,noir);
    TV.set_pixel(x+1,y+1,noir);
    TV.set_pixel(x+2,y+1,noir);
    TV.set_pixel(x+3,y+1,noir);
    TV.set_pixel(x+4,y+1,noir);
    TV.set_pixel(x-1,y+1,noir);
    TV.set_pixel(x-2,y+1,noir);
    TV.set_pixel(x-3,y+1,noir);
    TV.set_pixel(x,y+2,noir);
    TV.set_pixel(x+1,y+2,noir);
    TV.set_pixel(x+3,y+2,noir);
    TV.set_pixel(x-2,y+2,noir);
    TV.set_pixel(x+4,y+3,noir);
    TV.set_pixel(x-2,y+3,noir);
    TV.set_pixel(x+3,y+4,noir);
    TV.set_pixel(x-2,y+4,noir);
  }



  //reinitialisation de la variable
  //................................

  if (compteurISRglobal >=  (tempsDaffichage<<1) )
  {
    compteurISRglobal = 0;
  }

  //                   lecture ADC
  //........................................................

  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();


  //                 assignation de la lecture aDC 
  //..................................................................................
  int Value= (int)avgValue;
  tempsDaffichage=  Value>>5 ;


}


//                                                        Affiche des lignes
//..........................................................................................................................................
//******************************************************************************************************************************************
void perspective()
{
  //      notes
  //...............................
  /*
  pour simuler la perspetive le défilement des lignes (du bas vers le milieu de l'écran) ne doit pas être régulier. 
   En effet plus la ligne "remonte" plus elle doit "ralentir" sa remontée
   il doit y avoir un algorithme qui permet de calculer le ralentissement de cette ligne
   une fois calculer on peut stocker les valeurs dans un tableau
   */


  //    rappel des variables globales
  //..................................
  // pas de variables globale surilisées dans cettte section



  //      variables locales
  //................................
  //couleur d'affichage
  const uint8_t blanc = 1;
  const uint8_t noir = 0 ;

  //donne la résolution vertical
  const uint8_t resoVertical= TV.vres();

  //fixe l'abcsisse des segments de droite
  const uint8_t x0 = 0;
  const uint8_t x1 = 120;
  const uint8_t horizon = 40;




  //      affichage de l'horizon
  //.................................
  TV.draw_line(x0,horizon-1,x1,horizon-1,blanc);  




  //                                  lignes du milieu vers le bas
  //..............................................................................................................
  
  //      lignes principales
  //.....................................
  TV.select_font(font4x6);

  //explication du calcul:
  //le calcul en perpective ici utilise un "carré" "x^2" "x*x"
  //cette equation mathematique est assez proche de l'équation d'une perpspective!

  uint16_t p =  (  (  ct0 * ct0 ) >> 5 ) +   horizon;
  uint16_t pN = (   (  ( ct0-1) * (ct0-1)  ) >> 5 ) +   horizon;
  uint16_t q = (  (  ct1 * ct1 ) >> 5 ) +   horizon;
  uint16_t qN =(   (  ( ct1-1) * (ct1-1)  ) >> 5 ) +   horizon;
  uint16_t r = (  (  ct2 * ct2 ) >> 5 ) +   horizon;
  uint16_t rN =(   (  ( ct2-1) * (ct2-1)  ) >> 5 ) +   horizon;
  uint16_t s = (  (  ct3 * ct3 ) >> 5 ) +   horizon;
  uint16_t sN =(   (  ( ct3-1) * (ct3-1)  ) >> 5 ) +   horizon;
  uint16_t t = (  (  ct4 * ct4 ) >> 5 ) +   horizon;
  uint16_t tN =(   (  ( ct4-1) * (ct4-1)  ) >> 5 ) +   horizon;
  uint16_t u = (  (  ct5 * ct5 ) >> 5 ) +   horizon;
  uint16_t uN =(   (  ( ct5-1) * (ct5-1)  ) >> 5 ) +   horizon;
  uint16_t v = (  (  ct6 * ct6 ) >> 5 ) +   horizon;
  uint16_t vN =(   (  ( ct6-1) * (ct6-1)  ) >> 5 ) +   horizon;
  uint16_t w = (  (  ct7 * ct7 ) >> 5 ) +   horizon;
  uint16_t wN =(   (  ( ct7-1) * (ct7-1)  ) >> 5 ) +   horizon;



  TV.draw_line(x0,p,x1,p,blanc); 
  TV.draw_line(x0,pN,x1,pN,noir);

  TV.draw_line(x0,q,x1,q,blanc); 
  TV.draw_line(x0,qN,x1,qN,noir);

  TV.draw_line(x0,r,x1,r,blanc); 
  TV.draw_line(x0,rN,x1,rN,noir);

  TV.draw_line(x0,s,x1,s,blanc); 
  TV.draw_line(x0,sN,x1,sN,noir);

  TV.draw_line(x0,t,x1,t,blanc); 
  TV.draw_line(x0,tN,x1,tN,noir);

  TV.draw_line(x0,u,x1,u,blanc); 
  TV.draw_line(x0,uN,x1,uN,noir);
  /*
  TV.draw_line(x0,v,x1,v,blanc); 
   TV.draw_line(x0,vN,x1,vN,noir);
  /*
   TV.draw_line(x0,w,x1,w,blanc); 
   TV.draw_line(x0,wN,x1,wN,noir);
   */


  //      lignes relais
  //.....................................

  //de 40 à 168 
  uint16_t a =  (  (  cr0 * cr0 ) >> 5 ) +   horizon;
  uint16_t aN = (   (  ( cr0-1) * (cr0-1)  ) >> 5 ) +   horizon;
  uint16_t b =  (  (  cr1 * cr1 ) >> 5 ) +   horizon;
  uint16_t bN = (   (  ( cr1-1) * (cr1-1)  ) >> 5 ) +   horizon;
  uint16_t c =  (  (  cr2 * cr2 ) >> 5 ) +   horizon;
  uint16_t cN = (   (  ( cr2-1) * (cr2-1)  ) >> 5 ) +   horizon;
  uint16_t d =  (  (  cr3 * cr3 ) >> 5 ) +   horizon;
  uint16_t dN = (   (  ( cr3-1) * (cr3-1)  ) >> 5 ) +   horizon;  
  uint16_t e =  (  (  cr4 * cr4 ) >> 5 ) +   horizon;
  uint16_t eN = (   (  ( cr4-1) * (cr4-1)  ) >> 5 ) +   horizon;  
  uint16_t f =  (  (  cr5 * cr5 ) >> 5 ) +   horizon;
  uint16_t fN = (   (  ( cr5-1) * (cr5-1)  ) >> 5 ) +   horizon;  


  /*
  TV.print (10,10,cr0,DEC);
   TV.print (40,10,commutateurCompteur,DEC);
   */



  TV.draw_line(x0,a,x1,a,blanc); 
  TV.draw_line(x0,aN,x1,aN,noir);

  TV.draw_line(x0,b,x1,b,blanc); 
  TV.draw_line(x0,bN,x1,bN,noir);

  TV.draw_line(x0,c,x1,c,blanc); 
  TV.draw_line(x0,cN,x1,cN,noir);

  TV.draw_line(x0,d,x1,d,blanc); 
  TV.draw_line(x0,dN,x1,dN,noir);

  TV.draw_line(x0,e,x1,e,blanc); 
  TV.draw_line(x0,eN,x1,eN,noir);
  /*  
   TV.draw_line(x0,f,x1,f,blanc); 
   TV.draw_line(x0,fN,x1,fN,noir);
  /* 
   TV.draw_line(x0,g,x1,g,blanc); 
   TV.draw_line(x0,gN,x1,gN,noir);
   
   TV.draw_line(x0,h,x1,h,blanc); 
   TV.draw_line(x0,hN,x1,hN,noir);
   */



  
  
  
  
  
  //                                  lignes du milieu vers le haut
  //..............................................................................................................
  
  //      lignes principales
  //.....................................
  
  //explication du calcul:
  //le calcul en perpective ici utilise un "carré" "x^2" "x*x"
  //cette equation mathematique est assez proche de l'équation d'une perpspective!

  uint16_t g =  horizon  -(  (  ct0 * ct0 ) >> 5 )-2;
  uint16_t gN = horizon - (   (  ( ct0-1) * (ct0-1)  ) >> 5 ) -2 ;
  uint16_t h = horizon - (  (  ct1 * ct1 ) >> 5 ) -2 ;
  uint16_t hN = horizon - (   (  ( ct1-1) * (ct1-1)  ) >> 5 )-2 ;
  uint16_t i = horizon -(  (  ct2 * ct2 ) >> 5 ) -2;
  uint16_t iN = horizon -(   (  ( ct2-1) * (ct2-1)  ) >> 5 )-2 ;
  uint16_t j = horizon -(  (  ct3 * ct3 ) >> 5 )-2;
  uint16_t jN =horizon -(   (  ( ct3-1) * (ct3-1)  ) >> 5 ) -2;
  uint16_t k = horizon -(  (  ct4 * ct4 ) >> 5 ) -2;
  uint16_t kN = horizon -(   (  ( ct4-1) * (ct4-1)  ) >> 5 )-2;
  uint16_t l = horizon -(  (  ct5 * ct5 ) >> 5 ) -2;
  uint16_t lN =horizon -(   (  ( ct5-1) * (ct5-1)  ) >> 5 ) -2;
  /*uint16_t p = (  (  ct6 * ct6 ) >> 5 ) +   horizon;
  uint16_t pN =(   (  ( ct6-1) * (ct6-1)  ) >> 5 ) +   horizon;
  uint16_t q = (  (  ct7 * ct7 ) >> 5 ) +   horizon;
  uint16_t qN =(   (  ( ct7-1) * (ct7-1)  ) >> 5 ) +   horizon;
 */


  TV.draw_line(x0,g,x1,g,blanc); 
  TV.draw_line(x0,gN,x1,gN,noir);

  TV.draw_line(x0,h,x1,h,blanc); 
  TV.draw_line(x0,hN,x1,hN,noir);

  TV.draw_line(x0,i,x1,i,blanc); 
  TV.draw_line(x0,iN,x1,iN,noir);

  TV.draw_line(x0,j,x1,j,blanc); 
  TV.draw_line(x0,jN,x1,jN,noir);

  TV.draw_line(x0,k,x1,k,blanc); 
  TV.draw_line(x0,kN,x1,kN,noir);

  TV.draw_line(x0,l,x1,l,blanc); 
  TV.draw_line(x0,lN,x1,lN,noir);
  /*
  TV.draw_line(x0,v,x1,v,blanc); 
   TV.draw_line(x0,vN,x1,vN,noir);
  /*
   TV.draw_line(x0,w,x1,w,blanc); 
   TV.draw_line(x0,wN,x1,wN,noir);
   */





  //      lignes relais
  //.....................................

  //de 40 à 168 
  uint16_t m = horizon - (  (  cr0 * cr0 ) >> 5 )-2;
  uint16_t mN =horizon - (   (  ( cr0-1) * (cr0-1)  ) >> 5 ) -2;
  uint16_t n = horizon - (  (  cr1 * cr1 ) >> 5 )-2;
  uint16_t nN =horizon -(   (  ( cr1-1) * (cr1-1)  ) >> 5 )-2;
  uint16_t o = horizon - (  (  cr2 * cr2 ) >> 5 )-2;
  uint16_t oN =horizon - (   (  ( cr2-1) * (cr2-1)  ) >> 5 )-2;
  uint16_t x = horizon - (  (  cr3 * cr3 ) >> 5 )-2;
  uint16_t xN =horizon - (   (  ( cr3-1) * (cr3-1)  ) >> 5 ) -2;  
  uint16_t y = horizon - (  (  cr4 * cr4 ) >> 5 ) -2;
  uint16_t yN = horizon -(   (  ( cr4-1) * (cr4-1)  ) >> 5 ) -2;  
  uint16_t z = horizon - (  (  cr5 * cr5 ) >> 5 ) -2;
  uint16_t zN = horizon -(   (  ( cr5-1) * (cr5-1)  ) >> 5 ) -2;  


  TV.draw_line(x0,m,x1,m,blanc); 
  TV.draw_line(x0,mN,x1,mN,noir);

  TV.draw_line(x0,n,x1,n,blanc); 
  TV.draw_line(x0,nN,x1,nN,noir);

  TV.draw_line(x0,o,x1,o,blanc); 
  TV.draw_line(x0,oN,x1,oN,noir);

  TV.draw_line(x0,x,x1,x,blanc); 
  TV.draw_line(x0,xN,x1,xN,noir);

  TV.draw_line(x0,y,x1,y,blanc); 
  TV.draw_line(x0,yN,x1,yN,noir);
  /*  
   TV.draw_line(x0,f,x1,f,blanc); 
   TV.draw_line(x0,fN,x1,fN,noir);
  /* 
   TV.draw_line(x0,g,x1,g,blanc); 
   TV.draw_line(x0,gN,x1,gN,noir);
   
   TV.draw_line(x0,h,x1,h,blanc); 
   TV.draw_line(x0,hN,x1,hN,noir);
   */

  
  //                   lecture ADC
  //........................................................

  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();


  //         assignation de la lecture aDC au defilement de lignes
  //..................................................................................

  //valeur mini = 7
  //valeur max = 10


  int Value= (int)avgValue;

  if (Value <256)
  {
    ecart=10;
  }
  else if (( Value <512) && ( Value>=256) )
  {
    ecart=9;
  }
  else if ( (Value <768) && (Value >=512)  )
  {
    ecart=8;
  }
  else if (Value >=768)
  {
    ecart=7;
  }


}


//                                                         le commutateur             
//..........................................................................................................................................
//******************************************************************************************************************************************
void BoutonPoussoir() 
{
  //explication:
  //ici c'est la gestion du bouton poussoir mommentanné.
  //Cette fonction n'est qu'un équivalent du bistable analogique. Il sert de système anti rebond digitale. 
  //la réponse de la fonction n'est qu'une variable globale.



  //Rappel des variables globales

  //uint8_t variable_couleur = 0 ;
  //uint8_t memoireBoutonCouleur=0;
  //uint8_t fm_BoutonCouleur=0;



  //déclaration des variables
  uint8_t  lectureBPcouleur ;
  lectureBPcouleur = digitalRead( menuPin) ;

  //gestion du start stop
  if(!memoireBoutonCouleur && 1 == lectureBPcouleur )
  {
    memoireBoutonCouleur=HIGH;
    fm_BoutonCouleur=HIGH;
  }
  else if(0 == lectureBPcouleur)
  {
    memoireBoutonCouleur=LOW;
    fm_BoutonCouleur=LOW;
  }
  else if(memoireBoutonCouleur)
    fm_BoutonCouleur=LOW;
  // ici on peut se servir de fm_Bouton qui sera RAZ au prochain cycle
  if(fm_BoutonCouleur==HIGH)
  {
    variable_couleur ++ ;        
  }


  //limitation de variable_mult *******************Modifier si ajout d'un switch case Supplémentaire
  if (variable_couleur > 8)
  {
    variable_couleur = 0 ;
  }

}


//                                          "intro" fait descendre TVOlogo, rangé dans le void loop
//.........................................................................................................................................
//******************************************************************************************************************************************

void intro() {
  unsigned char w,l,wb;
  int index;
  w = pgm_read_byte(LOGORAZ);
  l = pgm_read_byte(LOGORAZ+1);
  if (w&7)
    wb = w/8 + 1;
  else
    wb = w/8;
  index = wb*(l-1) + 2;
  for ( unsigned char i = 1; i < l; i++ ) {
    TV.bitmap((TV.hres() - w)/2,0,LOGORAZ,index,w,i);
    index-= wb;

    TV.delay_frame(5); 
  }
  for (unsigned char i = 0; i < (TV.vres() - l)/2; i++) {
    TV.bitmap((TV.hres() - w)/2,i,LOGORAZ);

    TV.delay_frame(5); //              Descente Logo

  }

  TV.delay_frame(100);

  TV.clear_screen();
}


//                                                      oscilloscope 
//.........................................................................................................................................
//******************************************************************************************************************************************

/*
Oscilloscope Sub program based on the excellent work from Juerg Wullschleger:

TVout-Oscilloscope, v1.1
 
 Lets you display the voltage on analog pin A0 on your TV using the TVout Library.
 
 It displays the average, the minimal and the maximal voltage, and the sampling
 frequency. 100 sample points are displayed.
 
 The maximal sampling frequency is about 140kHz (on an Arduino Uno). Note that at higher
 frequencies, the measurement is less accurate.
 
 Add a button to digital pin 2 to scale the voltage range, and to digital pin 3
 to scale the sample range.
 
 
 v1.1: increased the max. sampling frequency from about 8kHz to about 140kHz.
 (Code to read analog input fast is based on code by Anatoly Kuzmenko.)
 
 
 (c) Juerg Wullschleger 2011.
 
 http://creativecommons.org/licenses/by/3.0/
 */



inline void oscilloscope()
{
  checkButtons();

  waitFor(sampleSize,avgValue);

  minValue = 1024; 
  maxValue = 0;
  long sum = 0;
  int i = 0;  
  startTime = micros();
  while(i < graphWidth){
    if(sampleSize <= 3){
      //for small sampleSize, sample first and then draw
      readFast(sampleSize);
      if (sampleTime > 0)
        sampleTime = (19 * sampleTime + micros() - startTime) / 20;
      else
        sampleTime = micros() - startTime;
      i=0;
      while(i < graphWidth){
        int val = data[i];
        minValue = min(minValue, val);
        maxValue = max(maxValue, val);
        if(sampleSize > 0)
          sum = sum + sampleSize*val;
        else
          sum = sum + val;
        drawData(i,val);
        i++;
      }
    }
    else{
      //for large sampleSize, do sample and draw at the same time.
      int minV = 1024, maxV = 0;
      volatile int val = 0;

      initADC(1);

      ADCSRA |= (1<<ADSC); // start reading first value
      for(int j = 0; j< sampleSize; j++){

        while(!(ADCSRA & 0x10));
        val = ADCL;
        val += (ADCH << 8);
        ADCSRA |= (1<<ADSC); // already start reading next value for more speed

        minV = min(minV,val);
        maxV = max(maxV,val);
        sum = sum + val;
      }

      resetADC();

      drawData(i,minV,maxV);
      minValue = min(minValue,minV);
      maxValue = max(maxValue,maxV);
      i++;

      if(checkButtons()){
        //button has been pressed. reset values.
        i = 0;
        startTime  = micros();
        minValue = 1024; 
        maxValue = 0;
        sum = 0;
      }
      if(i==graphWidth){
        if (sampleTime > 0)
          sampleTime = (9 * sampleTime + micros() - startTime) / 10;
        else
          sampleTime = micros() - startTime;
      }
    }
  }

  if(sampleSize > 0)
    avgValue =  sum / (graphWidth*sampleSize);
  else
    avgValue =  sum / graphWidth;

  printVolts();
  printSampleRate(); 

}



//                                                           VIPER MARK II
//.........................................................................................................................................
//******************************************************************************************************************************************

void vipersMarkII()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;

  uint8_t y;
  uint8_t yB;





  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 




  //          pattern
  //............................



  //set_pixel(x,y,color)
  y= random ( vRes) ;
  TV.set_pixel (120, y, blanc);
  TV.set_pixel (120-1, y, blanc);



  yB= random (vRes) ;                        //Imprime des ray de vitesse lumière
  TV.draw_line(0,yB,116-y,yB,blanc); 




  TV.bitmap(xMilieu+5,Value+10,VIPER); //imprime un Vaisseau Viper
  TV.bitmap(xMilieu+4,Value+6,VIPERBLACK); //imprime un cache noir au dessus du Viper 
  TV.bitmap(xMilieu+6,Value+23,VIPERBLACK); //imprime un cache noir en dessous du Viper 

  //mouvement de l'écran
  //shift(distance,direction)   direction =>  0=up 1 = down 2 = left 3=right 
  TV.shift(2,2) ; 


}
//                                                           Cpt Blood
//.........................................................................................................................................
//******************************************************************************************************************************************

void CptBlood()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;

  uint8_t y;
  uint8_t yB;





  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 




  //          pattern
  //............................


  
   TV.bitmap(xMilieu-15,Value,CPTBLOODBABY); //imprime un BB Cpt Blood
  
    
  //mouvement de l'écran
  //shift(distance,direction)   direction =>  0=up 1 = down 2 = left 3=right 
  TV.shift(1,2) ; 


}

//                                                           Mazinger
//.........................................................................................................................................
//******************************************************************************************************************************************

void Mazinger()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;


  uint8_t xB;




  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();
  


  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 




  //          pattern
  //............................

 TV.bitmap(42,0,MAZINGER); //imprime un Mazinger Z
 xB= random ( hRes) ;
 TV.draw_line(Value, xB, Value, 62,blanc); 
 
  TV.shift(1,0); 

}






//                                                        IMPLEMENTATION ADC
//                                                  renvoie "avgValue" de 0 à 1023 
//.........................................................................................................................................
//******************************************************************************************************************************************


void lectureADC()
{
  //                   lecture ADC
  //........................................................

  //Pour faire varier un peu l'affichage on implémente une lecture aDC


    minValue = 1024; 
  maxValue = 0;
  long sum = 0;
  int i = 0;  
  startTime = micros();
  while(i < graphWidth){
    if(sampleSize <= 3){
      //for small sampleSize, sample first and then draw
      readFast(sampleSize);
      if (sampleTime > 0)
        sampleTime = (19 * sampleTime + micros() - startTime) / 20;
      else
        sampleTime = micros() - startTime;
      i=0;
      while(i < graphWidth){
        int val = data[i];
        minValue = min(minValue, val);
        maxValue = max(maxValue, val);
        if(sampleSize > 0)
          sum = sum + sampleSize*val;
        else
          sum = sum + val;


        //           affichage de la "trace" facon oscillo   
        //......................................................................

        //drawData(i,val);

        i++;
      }
    }
    else{
      //for large sampleSize, do sample and draw at the same time.
      int minV = 1024, maxV = 0;
      volatile int val = 0;

      initADC(1);

      ADCSRA |= (1<<ADSC); // start reading first value
      for(int j = 0; j< sampleSize; j++){

        while(!(ADCSRA & 0x10));
        val = ADCL;
        val += (ADCH << 8);
        ADCSRA |= (1<<ADSC); // already start reading next value for more speed

        minV = min(minV,val);
        maxV = max(maxV,val);
        sum = sum + val;
      }

      resetADC();

      drawData(i,minV,maxV);
      minValue = min(minValue,minV);
      maxValue = max(maxValue,maxV);
      i++;

      if(checkButtons()){
        //button has been pressed. reset values.
        i = 0;
        startTime  = micros();
        minValue = 1024; 
        maxValue = 0;
        sum = 0;
      }
      if(i==graphWidth){
        if (sampleTime > 0)
          sampleTime = (9 * sampleTime + micros() - startTime) / 10;
        else
          sampleTime = micros() - startTime;
      }
    }
  }

  if(sampleSize > 0)
    avgValue =  sum / (graphWidth*sampleSize);
  else
    avgValue =  sum / graphWidth;

}









