 //***********************************************************************************************************************************************
//**                                                         RAZMASYNTH Vector
//**                                                     (c) Benoit Bender & Kévin Tkac 2016/2018
//**                                                    
//**
//***********************************************************************************************************************************************

/*INFO:
 
 
 -Ressource de la librairie: https://code.google.com/p/arduino-tvout/
 
 Cablage de l'arduino Uno:
 -Sortie Sync sur Digital 9 + resistance de 1K (Sur les petits écran de Telewizor c'est 750r)
 -Sortie Video sur Digital 7 + Resistance 470r (Chez moi c'est 330r) Les deux sorties sont sommés par ces resistances pour former le signal composite.
 -Sortie audio de la librairie TVout sur digital 11, pas utilisé.
 -Boutton de Scale sur port 2
 -Boutton de sample sur port 3
 -Boutton de Trig IN sur port 6
 */
//cablage du bouton poussoir
//...........................

// un resistor de 2.2k relie le +5V au pin D2 
//le commutateur momentanné est relié à une borne au pin D2
//l'autre borne du commutateur est relié à la masse
//un condo de 470n est branché en parallele du commutateur



//***********************************************************************************************************************************************
//**
//**                                                           BIBLIOTHEQUES
//**
//***********************************************************************************************************************************************

//attention les bibliothèque doivent être installées sur l'ordinateur hôte

//permet l'utilisation d'une télévision, écran LCD couleur.....
#include <TVout.h>
#include <fontALL.h>

//  permet la lecture des type de variable classique uint8_t et le reste
#include <stdint.h>

//permet d'utiliser l'EEPROM
#include <EEPROM.h>

//le logo d'initialisation + les Bitmaps d'animations
#include "BLACKSCREEN.h" //Display a Black screen for Trig Input
#include "LOGORAZ.h"
#include "POINT.h"
#include "VIPER.h"
#include "VIPERBLACK.h"
#include "CPTBLOODBABY.h"
#include "MAZINGER.h"
#include "LOGOVECTOR.h"

//***********************************************************************************************************************************************
//**
//**                                                     VARIABLES GLOBALES
//**
//***********************************************************************************************************************************************

//input pins     
const uint8_t inputPin = 0; // Le numero de la pin analo d'entrée CV
//const uint8_t scalePin = 2; // Le numero de la pin du boutton de scale utilisé aussi pour faire défiler le menu textuel, enlevé pour DU et Vector
const uint8_t samplePin = 3; // Le numero de la pin du boutton de sample urilisé aussi pour valider le choix du sous programme dans le menu textuel
const uint8_t menuPin = 4; //le numéro du pin pour la selection dans le menu
const uint8_t trigIn = 6; //le numéro du pin pour l'entrée Trig

//pour les boutons poussoirs
uint8_t variable_couleur = 0 ;     
uint8_t memoireBoutonCouleur=0;
uint8_t fm_BoutonCouleur=0;

//
int modeVector = 0; // pour le switch case de Vector
int valVector = 0; // Pin 13 HIGH/LOW Status
int butStateVector = 0; // Last Button State
int modeStateVector = 0; // Last Mode State

//Pour l'oscilloscope                                         
const int graphOffsetX = 6;      //notes: avant 10
const int graphOffsetY = 96;     //notes: avant 94
const int graphWidth = 102;      //notes: avant 99
const int graphHeight = 96;     //notes: avant 86

const int upperGraphLimit = graphOffsetY + 1;
const int lowerGraphLimit = graphOffsetY - graphHeight;

int scaleFactor = 1024/graphHeight+1;
int offset = 0;
int sampleSize = -4;
// if sampleSize <= 0, it denotes the sampling speed.

int minValue, maxValue, avgValue;
long startTime = 0, sampleTime = 0;

//used if sampleSize <=3 to draw faster
int data[graphWidth];
uint8_t dispY[graphWidth];

// pour la gestion de la bibliothèque TVout
TVout TV;

//pour l'appel une fois du dessin du rectangle+echelle
uint8_t premiereFoisRectangle ;


//pour le timer 0
//compteur principal ct comme compteur timer, sur DU passé de uint16_t à uint8_t
uint8_t ct0 ;
uint8_t ct1 ;
uint8_t ct2 ;
uint8_t ct3 ;
uint8_t ct4 ;
uint8_t ct5 ;
uint8_t ct6 ;
uint8_t ct7 ;

//compteur relair cr
uint8_t cr0 ;
uint8_t cr1 ;
uint8_t cr2 ;
uint8_t cr3 ;
uint8_t cr4 ;
uint8_t cr5 ;
uint8_t cr6 ;
uint8_t cr7 ;

//commutateur des compteur relais
//intervient en ISR et en loop
uint8_t commutateurCompteur=0;

//permet une gestion du defilement des lignes
uint8_t ecart;

//compteur en timer0 à la fréquence réelle
uint16_t compteurISRglobal ;

uint8_t blackScreen;         // variable for reading the Trig Input status //************************************


  volatile int x1 = 0;  //Volatile pour Pattern Vecteur Y
  volatile int y1 = 0;
  volatile int x2 = 0;
  volatile int y2 = 0;


//***********************************************************************************************************************************************
//**
//**                                                    INITIALISATION                                                          
//**
//***********************************************************************************************************************************************

void setup()
{                                         
  //    TIMER 0  61Hz
  //......................

  cli();//stop interrupts

  TCCR0A = 0;// set entire TCCR2A register to 0
  TCCR0B = 0;// same for TCCR2B
  TCNT0  = 0;//initialize counter value to 0
  // set compare match register for 2khz increments
  OCR0A = 255;// = (must be <256)
  // turn on CTC mode
  TCCR0A |= (1 << WGM01);
  // Set CS10 and CS12 bits for 1024 prescaler
  TCCR0B |= (1 << CS12) | (1 << CS10);  
  // enable timer compare interrupt
  TIMSK0 |= (1 << OCIE0A);

  //f=arduino clock sppeed/ ( prescaler * (compar match register +1) ) = 61.0351Hz
  
  //pour les boutons poussoirs 
  //pinMode(scalePin, INPUT);               // Initialize la pin du boutton comme une entree, enlevé pour DU et Vector
  pinMode(samplePin, INPUT);              // Initialize la pin du boutton comme une entree
  pinMode(menuPin, INPUT);                // Initialize la pin du boutton comme une entree
  pinMode(trigIn, INPUT);                // Initialize la pin de l'entrée Trig une entree

  //initialisation de la bibliothèque TV: NTSC ou PAL
  // TV.begin(PAL,120,96);                                 

  TV.begin(NTSC,120,96);                                     //°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°Reso changée NTSC 120,96

  intro(); //Appelle de la fonction ""intro"                                                                                            °°°INTRO°°°

  //pour l'oscilloscope sur DU fonction cleargraph et drawscale enlevé. Cleargraph remplacé par fonction blackscreen du debug switch

  //selection du pattern de démarrage en EEPROM
  variable_couleur = EEPROM.read(0);
  
   //ou par choix programmé: 
 //variable_couleur=0; //permet de démarrer ou on veut dans le switch case °°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°°

  //mise ne route TIMER 0
  sei();

}


//***********************************************************************************************************************************************
//**
//**                                                    INITIALISATION DE L OSCILLOSCOPE
//**
//**                                 **********     OSCILLOSCOPE       fonction secondaire      **********
//**
//**
//***********************************************************************************************************************************************



//..............................................................................
//                                  initADC
void initADC(int speed)
{
  // speed == 1: normal analogread.
  // speed in [-4,0]: faster (-4 is the fastest)

  ADCSRA = 0x87; //default
  if ( speed >= 1 ) ADCSRA = 0x87;   // turn on adc, freq  = 1/128,  125 kHz. 
  if ( speed == 0 ) ADCSRA = 0x86;   // turn on adc, freq  = 1/64,   250 kHz. 
  if ( speed == -1 ) ADCSRA = 0x85;   // turn on adc, freq  = 1/32,   500 kHz. 
  if ( speed == -2 ) ADCSRA = 0x84;   // turn on adc, freq  = 1/16 ,    1 MHz. 
  if ( speed == -3 ) ADCSRA = 0x83;   // turn on adc, freq  = 1/8,      2 MHz. 
  if ( speed == -4 ) ADCSRA = 0x82;   // turn on adc, freq  = 1/4,      4 MHz. 
  // does not seem to work:
  // if ( speed == -5 ) ADCSRA = 0x81;   // turn on adc, freq  = 1/2,      8 MHz.  

  ADMUX = 0x40;
  ADCSRA |= (1<<ADSC);
  while(!(ADCSRA & 0x10));
}

//..............................................................................
//                                    resetADC
void resetADC()
{
  ADCSRA = 0x00;
}

//..............................................................................
//                                 readFast
void readFast(int speed)
{
  // fills array with data on analog-pin 0.
  // speed >= 1: sample "speed" times and take the adverage.
  // speed in [-4,0]: read faster (-4 is the fastest)

  int * data_ptr = data; // pointer to array

  volatile char i = graphWidth; // iterator in array, init with array size
  volatile int val = 0, sum;

  initADC(speed);

  startTime = micros(); // set startTime here, to get a more accurate rate
  if(speed > 1){
    ADCSRA |= (1<<ADSC); // start reading the first value
    do{
      sum = 0;
      for(int j=0; j<speed; j++){
        while(!(ADCSRA & 0x10)); // wait for output
        val = ADCL;  // read out low data bits
        val += (ADCH << 8); // read out high data bits
        ADCSRA |= (1<<ADSC); // already start reading the next value for more speed
        sum += val;
      }
      *data_ptr++ = sum / speed;
    }
    while(--i);
  }
  else{
    ADCSRA |= (1<<ADSC); // start reading the first value
    do{
      while(!(ADCSRA & 0x10));
      val = ADCL;
      val += (ADCH << 8);
      ADCSRA |= (1<<ADSC); // already start reading the next value for more speed
      *data_ptr++ = val;
    }
    while(--i);
  }
  resetADC();
}

//..............................................................................
//                                       waitFor
void waitFor(int speed, int threshold)
{
  volatile int val = 0;

  initADC(speed);

  boolean wasLow = false;
  ADCSRA |= (1<<ADSC); // start reading first value
  for(int w=0 ; w<400 ; w++){
    while(!(ADCSRA & 0x10));
    val = ADCL;
    val += (ADCH << 8);
    ADCSRA |= (1<<ADSC); // already start reading next value for more speed
    if(val < threshold){
      wasLow = true;
    }
    else if((val > threshold) && wasLow){
      break;
    }
  }
  resetADC();
}

//..............................................................................
//                                       drawPixel
void drawPixel(int x, int y, char c)
{
  if((y > lowerGraphLimit) && (y < upperGraphLimit)){
    TV.set_pixel(x,y,c);
     TV.set_pixel(x,y-1,c); // Trace doublé en y, donc plus épaisse
  
  }
}

int val2y(int val){
  return graphOffsetY - val/scaleFactor - offset;
}


//..............................................................................
//                               drawData
void drawData(int i, int val)
{
  // we use dispY[i] to store the displayed pixel. so we only need to
  // erase that pixel, and not the whole column.
  int y = val2y(val);
  int x = i + graphOffsetX;
  drawPixel(x,dispY[i],0);
  drawPixel(x,y,1);
  dispY[i] = y;
}

//..............................................................................
//                              drawData
void drawData(int i, int v1, int v2)
{
  int u = val2y(v1);
  int v = val2y(v2);
  int x = i + graphOffsetX;
  TV.draw_column(x, graphOffsetY, graphOffsetY - graphHeight + 1, 0);
  u = min(max(u,lowerGraphLimit),upperGraphLimit);
  v = min(max(v,lowerGraphLimit),upperGraphLimit);
  TV.draw_column(x, u, v, 1);
}

//..............................................................................
//                              changeSampleSize
void changeSampleSize()
{
  if(sampleSize<5)
    sampleSize = sampleSize + 1;
  else
    //sampleSize = sampleSize * 2;
    sampleSize = sampleSize * 8 / 5;
  if(sampleSize > 300){
    sampleSize = -5;
  }
  sampleTime = 0;
  
  TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir
}

//..............................................................................                        
//                           PAS DANS UNE FONCTION
//                   lecture de bouton poussoir archaïque
boolean sampleButtonPressed = false;
boolean checkButtons()
{

  if(digitalRead(samplePin) == HIGH){
    sampleButtonPressed = true;
  }
  else{
    if(sampleButtonPressed){
      changeSampleSize();
      sampleButtonPressed = false;
      return true;
    }
  }
  return false;
}



//***********************************************************************************************************************************************
//**
//**                                                    TIMER 0
//**
//***********************************************************************************************************************************************


ISR(TIMER0_COMPA_vect)
{

  //    rappel variables globales
  //.................................
  //détermine l'écartement entre les différente ligne 
  //max = 10
  //mini = 7
  //c'est un uint8_t    "ecart"
  // en uint16 compteurISRglobal


  //  variables locales
  //..........................
  static uint8_t ISRcompteur ;
  ISRcompteur ++;
  const uint8_t compareMatchRegister = 2; 



  //pour une autres fonction d'affichage
  //.......................................
  compteurISRglobal ++ ;


  //   DIVISION DE FREQUENCE  
  //.................................
  //ici on virtualise le timer en implémentant un timer dans le timer.... :)

  if (ISRcompteur >  compareMatchRegister )
  {
    //A partir de là tout est virtualisé !!    :)
    // differents compteur sur timer 0
    //ct0 est le compteur "maitre" c'est de lui que viennent les autres compteurs
    ct0 ++ ;

    //          explications
    //........................................

    //il y a 2 groupes de compteurs, u ngroupe principal et un groupe de relais
    //le groupe principal est composé de 8 compteurs ct0 à ct7 (ct comme compteur timer)
    //ceux ci démarrre à interval régulier "ecart"
    //ces  compteurs sont remis à "0" quand le premier atteint la reinitialisation du Compteur "reInitialisation"

    //le groupe de relai est composé de compteurs; il servent à combler les "trous" laissés par la remise à zero des compteur du groupe principal
    //ils commence quand le "commutateurCompteur" passe à "1"
    //ils sont reinitialisé quand le dernier compteur atteint le "reInitialisation"  
    //ils sont synchronisés au groupe principal quand démarre le un compteur principal  
    //ils sont nommés cr (cr comme compteur relais) 

    //  l'initialisation des compteur principaux et compteur relais pilote
    //......................................................................
    if (ct0 > ecart*1)
    {
      ct1 ++;

    }
    if (ct0 > ecart*2)
    {
      ct2 ++;

    }
    if (ct0 > ecart*3)
    {
      ct3 ++;  

    }
    if (ct0 > ecart*3)
    {
      ct4 ++;  

    }
    if (ct0 > ecart*4)
    {
      ct5 ++; 
      commutateurCompteur=1;  
    }
    if (ct0 > ecart*5)
    {
      ct6 ++; 

    }
    if (ct0 > ecart*6)
    {
      ct7 ++; 

    } 

    //  l'initialisation des compteur  relais pilote
    //....................................................
    if (1 == commutateurCompteur)
    {
      cr0 ++;
    }
    if (cr0 > ecart*1)
    {
      cr1 ++;
    }
    if (cr0 > ecart*2)
    {
      cr2 ++;
    }
    if (cr0 > ecart*3)
    {
      cr3 ++;
    }
    if (cr0 > ecart*4)
    {
      cr4 ++;
    }
    if (cr0 > ecart*5)
    {
      cr5 ++;
    }

    // ré initialisation des variables
    //...................................

    //il faut reajuster le delai minimal lorsque l'on touche à l'"ecart" entre les lignes 
    //c'est la raison pour laquelle on multiple ecart par 4 avec "ecart<<2"
    //on ne peut dépasser une valeur maximal sinon les ligne viennent de tout en haut de l'écran
    //par calcul on obtient le mini  ctx*ctx/32=40 quand y=90

    const uint8_t reInitialisation =  ( ( ecart << 2 ) + 44 );

    if (ct0 >=   reInitialisation  )
    {
      ct0=0;  
      ct1=0;
      ct2=0;
      ct3=0;  
      ct4=0;
      ct5=0;
    }

    if ( ( cr0 >= reInitialisation )   )
    {
      cr0=0; 
      cr1=0;
      cr2=0;
      cr3=0;
      cr4=0;
      cr5=0;
      commutateurCompteur=0; 
    }

    //remise à zero du compteur 60Hz
    ISRcompteur = 0;

  }
}




//***********************************************************************************************************************************************
//**
//**                                                    PROGRAMME PRINCIPAL
//**
//***********************************************************************************************************************************************

void loop ( )
{



  //lecture du bouton poussoir de selection de programme 
  //renvoit une valeur "variable_couleur" de 0 à ? pour le switch case 
  // +++Pensez à augmenter le nombre dans "limitation de variable_mult" du code de commutateur, si ajout de switch case!!!

  BoutonPoussoir() ; 



  //                  SELECTION AFFICHAGE
  //***************************************************************                               SWITCH/CASE

  switch(variable_couleur)
    //switch(inputPinState)

  {

  case 0:

 vectorHorizonRoom();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    //ré initialisation de la variable
   // premiereFoisClearScreen = 1;

    break;


  case 1:

    //oscilloscope
 TV.clear_screen();
 
    oscilloscope();

    //ré initialisation de la variable

    break;

  case 2:

    vectorHorizon();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;

  case 3:

   Brett1();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus
    
    break;

  case 4:
    vipersMarkII();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;

  case 5:

    CptBlood();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;

  case 6:

    Mazinger();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus


    break;

  case 7:

     warp();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;


  case 8:

    ligneA();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;

   case 9:


     snake();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;   
    
  case 10:

  PointJPQuadrachromic();
   sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;    
    
  case 11:


   PointJP3marches();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;    

  case 12:

    // PointJPSolo(); remplacé par Gros pixels
    
      GrosPixels();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;
    
  case 13:


    Point();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break; 
    
   case 14:

 // derezzed();
   perspective();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus


    break;
 
  case 15:


   // Point2(); remplacé par perspective
  
//perspective();
     derezzed();
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;   
    
    
  case 16:

   Point3();
  
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;

  case 17:

    ligneA();
  
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

 
  case 18:
 
    ligneB();
  
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break;
    
     case 19:
 
    vecteurY();
  
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break; 
    
     case 20:
 
    tenPointAlgorythme();
  
    sampleSize = -4; //Reset le sample size et debug le bouton des menus

    break; 
    

  default:
    // Pas normal, on repart au début
    variable_couleur = 0;
  }

}


//************************************************************************************************************************************************************************************************
//**
//**                                                    LOGICIELS SECONDAIRES
//**
//************************************************************************************************************************************************************************************************





//****************************************************BASIC: INTRO + OSCILLOSCPE*******************************************************************
//*************************************************************************************************************************************

//                                          "intro" fait descendre TVOlogo, rangé dans le void loop
//.........................................................................................................................................
//******************************************************************************************************************************************

void intro() {
  unsigned char w,l,wb;
  int index;
  w = pgm_read_byte(LOGORAZ);
  l = pgm_read_byte(LOGORAZ+1);


  if (w&7)
    wb = w/8 + 1;
  else
    wb = w/8;
  index = wb*(l-1) + 2;
  for ( unsigned char i = 1; i < l; i++ ) {
    TV.bitmap((TV.hres() - w)/2,0,LOGORAZ,index,w,i);
    index-= wb;

    TV.delay_frame(5); 
  }
  for (unsigned char i = 0; i < (TV.vres() - l)/2; i++) {
    TV.bitmap((TV.hres() - w)/2,i,LOGORAZ);

    TV.delay_frame(5); //              Descente Logo

  }

  TV.delay_frame(100);


 TV.bitmap(7,60,LOGOVECTOR);

  TV.delay_frame(200);

  TV.clear_screen();
}


//                                                      Oscilloscope 
//.........................................................................................................................................
//******************************************************************************************************************************************


/*
Oscilloscope Sub program based on the excellent work from Juerg Wullschleger:
 
 TVout-Oscilloscope, v1.1
 
 Lets you display the voltage on analog pin A0 on your TV using the TVout Library.
 
 It displays the average, the minimal and the maximal voltage, and the sampling
 frequency. 100 sample points are displayed.
 
 The maximal sampling frequency is about 140kHz (on an Arduino Uno). Note that at higher
 frequencies, the measurement is less accurate.
 
 Add a button to digital pin 2 to scale the voltage range, and to digital pin 3
 to scale the sample range.
 
 
 v1.1: increased the max. sampling frequency from about 8kHz to about 140kHz.
 (Code to read analog input fast is based on code by Anatoly Kuzmenko.)
 
 
 (c) Juerg Wullschleger 2011.
 
 http://creativecommons.org/licenses/by/3.0/
 */

inline void oscilloscope()

{
  blackScreen = digitalRead(trigIn);

  if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

    {
      checkButtons();
      
     waitFor(sampleSize,avgValue);

      minValue = 1024; 
      maxValue = 0;
      long sum = 0;
      int i = 0;  
      startTime = micros();
      while(i < graphWidth){
        if(sampleSize <= 3){
          //for small sampleSize, sample first and then draw
          readFast(sampleSize);
          if (sampleTime > 0)
            sampleTime = (19 * sampleTime + micros() - startTime) / 20;
          else
            sampleTime = micros() - startTime;
          i=0;
          while(i < graphWidth){
            int val = data[i];
            minValue = min(minValue, val);
            maxValue = max(maxValue, val);
            if(sampleSize > 0)
              sum = sum + sampleSize*val;
            else
              sum = sum + val;
            drawData(i,val);
            i++;
          }
        }
        else{
          //for large sampleSize, do sample and draw at the same time.
          int minV = 1024, maxV = 0;
          volatile int val = 0;

          initADC(1);

          ADCSRA |= (1<<ADSC); // start reading first value
          for(int j = 0; j< sampleSize; j++){

            while(!(ADCSRA & 0x10));
            val = ADCL;
            val += (ADCH << 8);
            ADCSRA |= (1<<ADSC); // already start reading next value for more speed

            minV = min(minV,val);
            maxV = max(maxV,val);
            sum = sum + val;
          }

          resetADC();

          drawData(i,minV,maxV);
          minValue = min(minValue,minV);
          maxValue = max(maxValue,maxV);
          i++;

          if(checkButtons()){
            //button has been pressed. reset values.
            i = 0;
            startTime  = micros();
            minValue = 1024; 
            maxValue = 0;
            sum = 0;
          }
          if(i==graphWidth){
            if (sampleTime > 0)
              sampleTime = (9 * sampleTime + micros() - startTime) / 10;
            else
              sampleTime = micros() - startTime;
          }
        }
      }

      if(sampleSize > 0)
        avgValue =  sum / (graphWidth*sampleSize);
      else
        avgValue =  sum / graphWidth;

    //  printVolts(); printSampleRate(); drawScale(); Supprimés sur DU et Vector!!!!

    }


  } 
  else {    // et si condition de trig Input Low n'est pas remplie:

    TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir Trig IN

  }
}




//                                                         Le commutateur pour bouton Poussoir Menu       
//..........................................................................................................................................
//******************************************************************************************************************************************
void BoutonPoussoir() 
{
  //explication:
  //ici c'est la gestion du bouton poussoir mommentanné.
  //Cette fonction n'est qu'un équivalent du bistable analogique. Il sert de système anti rebond digitale. 
  //la réponse de la fonction n'est qu'une variable globale.



  //Rappel des variables globales

  //uint8_t variable_couleur = 0 ;
  //uint8_t memoireBoutonCouleur=0;
  //uint8_t fm_BoutonCouleur=0;



  //déclaration des variables
  uint8_t  lectureBPcouleur ;
  lectureBPcouleur = digitalRead( menuPin) ;

  //gestion du start stop
  if(!memoireBoutonCouleur && 1 == lectureBPcouleur )
  {
    memoireBoutonCouleur=HIGH;
    fm_BoutonCouleur=HIGH;
  }
  else if(0 == lectureBPcouleur)
  {
    memoireBoutonCouleur=LOW;
    fm_BoutonCouleur=LOW;
  }
  else if(memoireBoutonCouleur)
    fm_BoutonCouleur=LOW;
  // ici on peut se servir de fm_Bouton qui sera RAZ au prochain cycle
  if(fm_BoutonCouleur==HIGH)
  {
    variable_couleur ++ ;        
  }


  //limitation de variable_mult **********************************************Modifier si ajout d'un switch case Supplémentaire*************************
  if (variable_couleur > 20)
  {
    variable_couleur = 0 ;
  }

//écriture en EEPROM 
  if ( ( EEPROM.read(0) ) != (variable_couleur ) )
  {
    EEPROM.write(0, (variable_couleur ) );
  }




}





//                                                        IMPLEMENTATION ADC
//                                                  renvoie "avgValue" de 0 à 1023 
//.........................................................................................................................................
//******************************************************************************************************************************************


void lectureADC()
{
  //                   lecture ADC
  //........................................................

  //Pour faire varier un peu l'affichage on implémente une lecture aDC


    minValue = 1024; 
  maxValue = 0;
  long sum = 0;
  int i = 0;  
  startTime = micros();
  while(i < graphWidth){
    if(sampleSize <= 3){
      //for small sampleSize, sample first and then draw
      readFast(sampleSize);
      if (sampleTime > 0)
        sampleTime = (19 * sampleTime + micros() - startTime) / 20;
      else
        sampleTime = micros() - startTime;
      i=0;
      while(i < graphWidth){
        int val = data[i];
        minValue = min(minValue, val);
        maxValue = max(maxValue, val);
        if(sampleSize > 0)
          sum = sum + sampleSize*val;
        else
          sum = sum + val;

        //           affichage de la "trace" facon oscillo   
        //......................................................................

        //drawData(i,val);

        i++;
      }
    }
    else{
      //for large sampleSize, do sample and draw at the same time.
      int minV = 1024, maxV = 0;
      volatile int val = 0;

      initADC(1);

      ADCSRA |= (1<<ADSC); // start reading first value
      for(int j = 0; j< sampleSize; j++){

        while(!(ADCSRA & 0x10));
        val = ADCL;
        val += (ADCH << 8);
        ADCSRA |= (1<<ADSC); // already start reading next value for more speed

        minV = min(minV,val);
        maxV = max(maxV,val);
        sum = sum + val;
      }

      resetADC();

      drawData(i,minV,maxV);
      minValue = min(minValue,minV);
      maxValue = max(maxValue,maxV);
      i++;

      if(checkButtons()){
        //button has been pressed. reset values.
        i = 0;
        startTime  = micros();
        minValue = 1024; 
        maxValue = 0;
        sum = 0;
      }
      if(i==graphWidth){
        if (sampleTime > 0)
          sampleTime = (9 * sampleTime + micros() - startTime) / 10;
        else
          sampleTime = micros() - startTime;
      }
    }
  }

  if(sampleSize > 0)
    avgValue =  sum / (graphWidth*sampleSize);
  else
    avgValue =  sum / graphWidth;

}

//**
//************************************************************************************************************************************************************************************************

//                                                              Patterns

//************************************************************************************************************************************************************************************************

//***************************************************************************************************************************

//                                                           Vecteur Y
//.........................................................................................................................................
//******************************************************************************************************************************************

void vecteurY()
{
  //  Pas de variables locales
  //......................

  

  lectureADC();

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
   
     // TV.invert(); //Invert the data on the screen. ;
  TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir pour réinitialiser l'affichage
  modeVector = 0; //remet à zéro le switch case
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern:

  {


                                     //modeVector **********************************  ************** *******************
//int modeVector = 0; // pour le switch case de Vector
//int valVector = 0; //  HIGH/LOW Status
//int butStateVector = 0; // Last Button State
//int modeStateVector = 0; // Last Mode State



valVector = digitalRead(trigIn);

  // If we see a change in button state, increment mode value
 if (valVector != butStateVector && valVector == LOW){
 modeVector++;
 }
 
 butStateVector = valVector; // Keep track of most recent button state
 
 // No need to keep setting pins *every* loop
 if (modeStateVector != modeVector){
 
 // If no keys have been pressed yet don't execute the switch code below if (mode != 0) 
   
 switch ( modeVector ) {   


 default:
 modeVector = 0; // Mode par defaut à l'initialisation, c'est à dire quand la valeur de l'int "mode" est à 0

     TV.draw_line(x1,y1,x2,y2,WHITE);
 //break;

 
 
 case 1:
 
 modeVector = 1;  // Mode 1 par défault dès premier appui sur Switch

       x1=x2;               

 break;
 

case 2:

        y1=y2;              
 break;
 

 case 3:
    x2= avgValue/8;

 break;
 
 
 case 4:
 
     y2= avgValue/8;

 } // end switch


 } // end of ModeState check
 //modeStateVector = modeVector; // Keep track of mode recent mode value

}

  }


//accolade de samplepin
 }




//                                                                Vector Horizon -Send Gate to Trig, CV to move the path, A button turn black the screen
//.........................................................................................................................................
//******************************************************************************************************************************************

void vectorHorizon()
{
  //    variables locales
  //......................

  uint8_t x;
  uint8_t y;
  uint8_t tempsDaffichage;


  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();


  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 16
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  tempsDaffichage = Value;
  if (tempsDaffichage < 3)
  {
    tempsDaffichage = 2;
  }
 
 

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
   
     // TV.invert(); //Invert the data on the screen. ;
   TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir pour réinitialiser l'affichage
          modeVector = 0; //remet à zéro le switch case

  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern:

  {
  //  blackScreen = digitalRead(trigIn);

   // if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

      //          pattern
      //............................

int xA= 0;
int xB= 119;

  int x1 = 50;
  int y1= 30;
  int x2= 50;
  int y2= 30; 
  



                                     //modeVector **********************************  ************** *******************
//int modeVector = 0; // pour le switch case de Vector
//int valVector = 0; //  HIGH/LOW Status
//int butStateVector = 0; // Last Button State
//int modeStateVector = 0; // Last Mode State



valVector = digitalRead(trigIn);

  // If we see a change in button state, increment mode value
 if (valVector != butStateVector && valVector == LOW){
 modeVector++;
 }
 
 butStateVector = valVector; // Keep track of most recent button state
 
 // No need to keep setting pins *every* loop
 if (modeStateVector != modeVector){
 
 // If no keys have been pressed yet don't execute the switch code below if (mode != 0) 
   
 switch ( modeVector ) {   


 default:
 modeVector = 0;
 // Mode par defaut à l'initialisation, c'est à dire quand la valeur de l'int "mode" est à 0
 //TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir pour réinitialiser l'affichage

          
// break;
 
 case 1:
 
 modeVector = 1;  // Mode 1 par défault dès premier appui sur Switch
    x1= x1+ Value/2;
    x2= x2+ Value/2;
  TV.draw_line(xA,y1,x2-8,y2,WHITE);                                //Horizon fixe
      TV.draw_line(x1-8,y1,x2-8,y2+8,WHITE);
           TV.draw_line(x1 -8,y1+8,x2+8,y2+8,WHITE);
                TV.draw_line(x1 +8,y1+8,x2+8,y2,WHITE);
                      TV.draw_line(x1 +8,y1,xB,y2,WHITE);
                      


 break;


case 2:
    x1= x1+ Value/2;
    x2= x2+ Value/2;
  TV.draw_line(xA,y1+4,x2-12,y2+4,WHITE);                               //y+4
      TV.draw_line(x1-12,y1+4,x2-12,y2+12,WHITE);
           TV.draw_line(x1 -12,y1+12,x2+12,y2+12,WHITE);
                TV.draw_line(x1 +12,y1+12,x2+12,y2+4,WHITE);
                      TV.draw_line(x1 +12,y1+4,xB,y2+4,WHITE);
                      
 break;

 case 3:
    x1= x1+ Value/2;
    x2= x2+ Value/2;
  TV.draw_line(xA,y1+10,x2-16,y2+10,WHITE);                               //y+4+2
      TV.draw_line(x1-16,y1+10,x2-16,y2+18,WHITE);
           TV.draw_line(x1 -16,y1+18,x2+16,y2+18,WHITE);
                TV.draw_line(x1 +16,y1+18,x2+16,y2+10,WHITE);
                      TV.draw_line(x1 +16,y1+10,xB,y2+10,WHITE); 
                      

 break;
 
 case 4:
    x1= x1+ Value/2;
    x2= x2+ Value/2;
  TV.draw_line(xA,y1+18,x2-22,y2+18,WHITE);                               //4+4 x-2/+2
      TV.draw_line(x1-22,y1+18,x2-22,y2+26,WHITE);
           TV.draw_line(x1 -22,y1+26,x2+22,y2+26,WHITE);
                TV.draw_line(x1 +22,y1+26,x2+22,y2+18,WHITE);
                      TV.draw_line(x1 +22,y1+18,xB,y2+18,WHITE); 
     
 break;
 
  case 5:
    x1= x1+ Value/2;
    x2= x2+ Value/2;
  TV.draw_line(xA,y1+28,x2-28,y2+28,WHITE);                               //+4+6 x+4
      TV.draw_line(x1-28,y1+28,x2-28,y2+36,WHITE);
           TV.draw_line(x1 -28,y1+36,x2+28,y2+36,WHITE);
                TV.draw_line(x1 +28,y1+36,x2+28,y2+28,WHITE);
                      TV.draw_line(x1 +28,y1+28,xB,y2+28,WHITE); 
     
 break;
 
  case 6:
    x1= x1+ Value/2;
    x2= x2+ Value/2;
  TV.draw_line(xA,y1+40,x2-34,y2+40,WHITE);                               //+4+8 x+6
      TV.draw_line(x1-34,y1+40,x2-34,y2+48,WHITE);
           TV.draw_line(x1 -34,y1+48,x2+34,y2+48,WHITE);
                TV.draw_line(x1 +34,y1+48,x2+34,y2+40,WHITE);
                      TV.draw_line(x1 +34,y1+40,xB,y2+40,WHITE);
                      
     
 break;
 
  case 7:
    x1= x1+ Value/2;
    x2= x2+ Value/2;
  TV.draw_line(xA,y1+54,x2-40,y2+54,WHITE);                               //+4+10 x+8
      TV.draw_line(x1-40,y1+54,x2-40,y2+62,WHITE);
           TV.draw_line(x1 -40,y1+62,x2+40,y2+62,WHITE);
                TV.draw_line(x1 +40,y1+62,x2+40,y2+54,WHITE);
                      TV.draw_line(x1 +40,y1+54,xB,y2+54,WHITE);
     
 break;

  case 8:
TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir pour réinitialiser l'affichage

     
 break;




 } // end switch

 } // end of "if mode = 1" check

}


  }


  //reinitialisation de la variable
  //................................
  //quand on atteint deux fois le temps d'affichage on reinitialise
  if (compteurISRglobal >=  (tempsDaffichage<<1) )
  {
    compteurISRglobal = 0;
  }

}



//                                                                Vector Horizon Room -Send Gate to Trig
//.........................................................................................................................................
//******************************************************************************************************************************************

void vectorHorizonRoom()
{
  //    variables locales
  //......................


  uint8_t tempsDaffichage;


  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();


  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 16
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  tempsDaffichage = Value;
  if (tempsDaffichage < 3)
  {
    tempsDaffichage = 2;
  }
  
{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
   // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir pour réinitialiser l'affichage
          modeVector = 0; //remet à zéro le switch case
    
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern:

  {

      //          pattern
      //............................
   //   x= random ( hRes) ;
   //  y= random ( vRes );  
      
   int x1 = 60;
  int y1= 30;
  int x2= 60;
  int y2= 30;     

      
        TV.shift(2,0); 

                                     //modeVector **********************************  ************** *******************
//int modeVector = 0; // pour le switch case de Vector
//int valVector = 0; // Pin 13 HIGH/LOW Status
//int butStateVector = 0; // Last Button State
//int modeStateVector = 0; // Last Mode State

valVector = digitalRead(trigIn);

 TV.draw_line(x1-60,y1-2,x2-6,y2-2,WHITE);                                //Horizon fixe
      TV.draw_line(x1-6,y1-2,x2-6,y2+6,WHITE);
           TV.draw_line(x1 -6,y1+6,x2+6,y2+6,WHITE);
                TV.draw_line(x1 +6,y1+6,x2+6,y2-2,WHITE);
                      TV.draw_line(x1 +6,y1-2,x2+60,y2-2,WHITE);

 TV.draw_line(x1-60,y1,x2-8,y2,WHITE);                                //Horizon fixe
      TV.draw_line(x1-8,y1,x2-8,y2+8,WHITE);
           TV.draw_line(x1 -8,y1+8,x2+8,y2+8,WHITE);
                TV.draw_line(x1 +8,y1+8,x2+8,y2,WHITE);
                      TV.draw_line(x1 +8,y1,x2+60,y2,WHITE);
                   
  TV.draw_line(x1-60,y1+4,x2-12,y2+4,WHITE);                               //y+4
      TV.draw_line(x1-12,y1+4,x2-12,y2+12,WHITE);
           TV.draw_line(x1 -12,y1+12,x2+12,y2+12,WHITE);
                TV.draw_line(x1 +12,y1+12,x2+12,y2+4,WHITE);
                      TV.draw_line(x1 +12,y1+4,x2+60,y2+4,WHITE);
            
  TV.draw_line(x1-60,y1+10,x2-16,y2+10,WHITE);                               //y+4+2
      TV.draw_line(x1-16,y1+10,x2-16,y2+18,WHITE);
           TV.draw_line(x1 -16,y1+18,x2+16,y2+18,WHITE);
                TV.draw_line(x1 +16,y1+18,x2+16,y2+10,WHITE);
                      TV.draw_line(x1 +16,y1+10,x2+60,y2+10,WHITE);                      

  TV.draw_line(x1-60,y1+18,x2-22,y2+18,WHITE);                               //4+4 x-2/+2
      TV.draw_line(x1-22,y1+18,x2-22,y2+26,WHITE);
           TV.draw_line(x1 -22,y1+26,x2+22,y2+26,WHITE);
                TV.draw_line(x1 +22,y1+26,x2+22,y2+18,WHITE);
                      TV.draw_line(x1 +22,y1+18,x2+60,y2+18,WHITE);                      
                      
  TV.draw_line(x1-60,y1+28,x2-28,y2+28,WHITE);                               //+4+6 x+4
      TV.draw_line(x1-28,y1+28,x2-28,y2+36,WHITE);
           TV.draw_line(x1 -28,y1+36,x2+28,y2+36,WHITE);
                TV.draw_line(x1 +28,y1+36,x2+28,y2+28,WHITE);
                      TV.draw_line(x1 +28,y1+28,x2+60,y2+28,WHITE); 

  TV.draw_line(x1-60,y1+40,x2-34,y2+40,WHITE);                               //+4+8 x+6
      TV.draw_line(x1-34,y1+40,x2-34,y2+48,WHITE);
           TV.draw_line(x1 -34,y1+48,x2+34,y2+48,WHITE);
                TV.draw_line(x1 +34,y1+48,x2+34,y2+40,WHITE);
                      TV.draw_line(x1 +34,y1+40,x2+60,y2+40,WHITE);

  TV.draw_line(x1-60,y1+54,x2-40,y2+54,WHITE);                               //+4+10 x+8
      TV.draw_line(x1-40,y1+54,x2-40,y2+62,WHITE);
           TV.draw_line(x1 -40,y1+62,x2+40,y2+62,WHITE);
                TV.draw_line(x1 +40,y1+62,x2+40,y2+54,WHITE);
                      TV.draw_line(x1 +40,y1+54,x2+60,y2+54,WHITE);

  // If we see a change in button state, increment mode value
 if (valVector != butStateVector && valVector == LOW){
 modeVector++;
 }
 
 butStateVector = valVector; // Keep track of most recent button state
 
 // No need to keep setting pins *every* loop
 if (modeStateVector != modeVector){
 
 // If no keys have been pressed yet don't execute the switch code below if (mode != 0) 
   
 switch ( modeVector ) {   


 default:
 modeVector = 0;
 // Mode par defaut à l'initialisation, c'est à dire quand la valeur de l'int "mode" est à 0
          
 break;
 
 case 1:
 
 modeVector = 1;  // Mode 1 par défault dès premier appui sur Switch

  TV.draw_line(x1-60,y1,x2-8,y2,BLACK);                                //Horizon fixe
      TV.draw_line(x1-8,y1,x2-8,y2+8,BLACK);
           TV.draw_line(x1 -8,y1+8,x2+8,y2+8,BLACK);
                TV.draw_line(x1 +8,y1+8,x2+8,y2,BLACK);
                      TV.draw_line(x1 +8,y1,x2+60,y2,BLACK);

 break;


case 2:

  TV.draw_line(x1-60,y1+4,x2-12,y2+4,BLACK);                               //y+4
      TV.draw_line(x1-12,y1+4,x2-12,y2+12,BLACK);
           TV.draw_line(x1 -12,y1+12,x2+12,y2+12,BLACK);
                TV.draw_line(x1 +12,y1+12,x2+12,y2+4,BLACK);
                      TV.draw_line(x1 +12,y1+4,x2+60,y2+4,BLACK);

 
 case 3:

  TV.draw_line(x1-60,y1+10,x2-16,y2+10,BLACK);                               //y+4+2
      TV.draw_line(x1-16,y1+10,x2-16,y2+18,BLACK);
           TV.draw_line(x1 -16,y1+18,x2+16,y2+18,BLACK);
                TV.draw_line(x1 +16,y1+18,x2+16,y2+10,BLACK);
                      TV.draw_line(x1 +16,y1+10,x2+60,y2+10,BLACK); 
 break;
 
 case 4:

  TV.draw_line(x1-60,y1+18,x2-22,y2+18,BLACK);                               //4+4 x-2/+2
      TV.draw_line(x1-22,y1+18,x2-22,y2+26,BLACK);
           TV.draw_line(x1 -22,y1+26,x2+22,y2+26,BLACK);
                TV.draw_line(x1 +22,y1+26,x2+22,y2+18,BLACK);
                      TV.draw_line(x1 +22,y1+18,x2+60,y2+18,BLACK); 
     
 break;
 
  case 5:

  TV.draw_line(x1-60,y1+28,x2-28,y2+28,BLACK);                               //+4+6 x+4
      TV.draw_line(x1-28,y1+28,x2-28,y2+36,BLACK);
           TV.draw_line(x1 -28,y1+36,x2+28,y2+36,BLACK);
                TV.draw_line(x1 +28,y1+36,x2+28,y2+28,BLACK);
                      TV.draw_line(x1 +28,y1+28,x2+60,y2+28,BLACK); 
     
 break;
 
  case 6:

  TV.draw_line(x1-60,y1+40,x2-34,y2+40,BLACK);                               //+4+8 x+6
      TV.draw_line(x1-34,y1+40,x2-34,y2+48,BLACK);
           TV.draw_line(x1 -34,y1+48,x2+34,y2+48,BLACK);
                TV.draw_line(x1 +34,y1+48,x2+34,y2+40,BLACK);
                      TV.draw_line(x1 +34,y1+40,x2+60,y2+40,BLACK);
     
 break;
 
  case 7:

  TV.draw_line(x1-60,y1+54,x2-40,y2+54,BLACK);                               //+4+10 x+8
      TV.draw_line(x1-40,y1+54,x2-40,y2+62,BLACK);
           TV.draw_line(x1 -40,y1+62,x2+40,y2+62,BLACK);
                TV.draw_line(x1 +40,y1+62,x2+40,y2+54,BLACK);
                      TV.draw_line(x1 +40,y1+54,x2+60,y2+54,BLACK);
     
 break;

 } // end switch


 } // end of "if mode = 1" check

}

  }


  //reinitialisation de la variable
  //................................
  //quand on atteint deux fois le temps d'affichage on reinitialise
  if (compteurISRglobal >=  (tempsDaffichage<<1) )
  {
    compteurISRglobal = 0;
  }

}



//accolade de samplepin
 //}


//                                                         INVADER LINE LONG
//..........................................................................................................................................
//******************************************************************************************************************************************
void invaderLineLong()
{

  //rappel variable global
  //.............................
  //compteurISRglobal compte en uint16 à 60 Hz


  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  uint8_t x  ;
  uint8_t y ;
  uint8_t pointX  ;
  const uint8_t blanc = 1;
  const uint8_t noir =0;
  static uint16_t tempsDaffichage ;


{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:


  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:
      //     affichage
      //...................
      x= random ( hRes) ;
      y= random ( vRes );
      pointX= random ( vRes );

      if (compteurISRglobal< (tempsDaffichage >> 3) )
      {

        TV.set_pixel(x,y,blanc); 
        TV.set_pixel(x+3,y,blanc); 
        TV.set_pixel(x+4,y,blanc); 
        TV.set_pixel(x+7,y,blanc); 
        TV.set_pixel(x+9,y,blanc); 
        TV.set_pixel(x+11,y,blanc); 
        TV.set_pixel(x+12,y,blanc);
        TV.set_pixel(x+19,y,blanc); 
        TV.set_pixel(x+20,y,blanc); 
        TV.set_pixel(x+24,y,blanc); 
        TV.set_pixel(x+27,y,blanc); 
        TV.set_pixel(x+30,y,blanc); 
        TV.set_pixel(x+34,y,blanc); 
        TV.set_pixel(x+40,y,blanc);
        TV.set_pixel(x+49,y,blanc); 
        TV.set_pixel(x+50,y,blanc); 
        TV.set_pixel(x+54,y,blanc); 
        TV.set_pixel(x+57,y,blanc); 
        TV.set_pixel(x+60,y,blanc); 
        TV.set_pixel(x+64,y,blanc); 
        TV.set_pixel(x+70,y,blanc);
      }
      else
      {
        //desaffichage
        TV.set_pixel(x,y,noir); 
        TV.set_pixel(x+3,y,noir); 
        
        TV.set_pixel(x+27,y,blanc); 
        TV.set_pixel(x+30,y,blanc); 
        TV.set_pixel(x+34,y,blanc); 

        TV.set_pixel(x+50,y,blanc); 
       
      }

      //reinitialisation de la variable
      //................................
      
      if (compteurISRglobal >=  (tempsDaffichage<<1) )
      {
        compteurISRglobal = 0;
      }

      //                   lecture ADC
      //........................................................

      //Pour faire varier un peu l'affichage on implémente une lecture aDC
      lectureADC();


      //                 assignation de la lecture aDC 
      //..................................................................................
      int Value= (int)avgValue;
      tempsDaffichage=  Value>>5 ;


    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir au dessus du Viper 

    }
  }

}
//accolade de samplepin
 }

//                                                           POINT
//.........................................................................................................................................
//******************************************************************************************************************************************

void Point()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t blanc = 1;


  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 


{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:

  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:


      //          pattern
      //............................

     TV.bitmap(0,0,POINT); //imprime une ligne "POINT"
         TV.bitmap(2,3,POINT); //imprime une ligne "POINT"
 
    
     TV.draw_line(0, Value+40, 118, Value+40, blanc); 
     
     
          TV.shift(1,0); 
 

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 

    }
  }

}
//accolade de samplepin
 }
 
//                                                           POINT 2 
//.........................................................................................................................................
//******************************************************************************************************************************************

void Point2()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres(); 
  const uint8_t blanc = 1;


  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();

  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:



  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:


      //          pattern
      //............................

    TV.bitmap(0,90,POINT); //imprime une ligne "POINT"
         TV.bitmap(2,93,POINT); //imprime une ligne "POINT"

    
    
     TV.draw_line(0, Value, 118, Value, blanc); 

     
          TV.shift(2,0); 

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir au dessus du Viper 
    }
  }

}
//accolade de samplepin
 }
 
//                                                           POINT 3
//.........................................................................................................................................
//******************************************************************************************************************************************

void Point3()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;

  uint8_t xB;

  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:


  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:


      //          pattern
      //............................

     TV.bitmap(0,0,POINT); //imprime une ligne "POINT"
         TV.bitmap(2,3,POINT); //imprime une ligne "POINT"

        TV.bitmap(2,93,POINT); //imprime une ligne "POINT"

        TV.draw_line(0, Value+40, 118, Value+40, blanc); 

          TV.shift(1,0);
    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir au dessus du Viper 

    }
  }

}
//accolade de samplepin
 }
 
//                                                                ligneA
//.........................................................................................................................................
//******************************************************************************************************************************************

void ligneA()
{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;
  const uint8_t yMilieu = vRes >> 1 ;
  const uint8_t blanc = 1;
  const uint8_t noir =0;
  uint8_t x;
  uint8_t y;
  uint8_t tempsDaffichage;
  static uint8_t premiereFois = 1;

  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 16
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  tempsDaffichage = Value;
  if (tempsDaffichage < 3)
  {
    tempsDaffichage = 2;
  }

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:




  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

      //          pattern
      //............................
      x= random ( hRes) ;
      y= random ( vRes );


      if (compteurISRglobal<tempsDaffichage)
      {   
     //   TV.draw_line(xMilieu,yMilieu,x,y,2);
          TV.draw_line(xMilieu,Value+10,x,Value+10,2);


        //TV.draw_line(xMilieu,yMilieu,x,y,noir);

      }
      else
      {
        //desaffichage
        //draw_circle(x,y,radius,color,fillcolor)    ATTENTION a ne pas dépasser un radius de plus de 50

        for (uint8_t i = 0 ; i<14; i++)
        {
          TV.draw_circle(xMilieu,yMilieu,i<<2,0,0);

        }
      }

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un écran noir pour simuler Gate IN
    }
  }


  //reinitialisation de la variable
  //................................
  //quand on atteint deux fois le temps d'affichage on reinitialise
  if (compteurISRglobal >=  (tempsDaffichage<<1) )
  {
    compteurISRglobal = 0;
  }

}
//accolade de samplepin
 }

//                                                                ligneB
//.........................................................................................................................................
//******************************************************************************************************************************************

void ligneB()
{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;
  const uint8_t yMilieu = vRes >> 1 ;
  const uint8_t blanc = 1;
  const uint8_t noir =0;
  uint8_t x;
  uint8_t y;
  uint8_t tempsDaffichage;
  static uint8_t premiereFois = 1;

  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 16
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  tempsDaffichage = Value;
  if (tempsDaffichage < 3)
  {
    tempsDaffichage = 2;
  }

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:



  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

      //          pattern
      //............................
      x= random ( hRes) ;
      y= random ( vRes );


      if (compteurISRglobal<tempsDaffichage)
      {   
     //   TV.draw_line(xMilieu,yMilieu,x,y,2);
          TV.draw_line(xMilieu,Value+10,x,Value+10,2);
          
     //     TV.draw_rect(10,y,100,Value,blanc); //draw_rect(x0,y0,width,height,color)

        //TV.draw_line(xMilieu,yMilieu,x,y,noir);

      }
      else
      {
        //desaffichage
        //draw_circle(x,y,radius,color,fillcolor)    ATTENTION a ne pas dépasser un radius de plus de 50

        for (uint8_t i = 0 ; i<14; i++)
        {
     //     TV.draw_circle(xMilieu,yMilieu,i<<2,0,0);
     
    //           TV.draw_rect(10,y,100,Value,noir); //draw_rect(x0,y0,width,height,color)
          
       TV.shift(1,1) ;   
          
          
        }
      }

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un écran noir pour simuler Gate IN
    }
  }


  //reinitialisation de la variable
  //................................
  //quand on atteint deux fois le temps d'affichage on reinitialise
  if (compteurISRglobal >=  (tempsDaffichage<<1) )
  {
    compteurISRglobal = 0;
  }

}
//accolade de samplepin
 }
//                                                        derezzed 
//..........................................................................................................................................          **********      **********
//******************************************************************************************************************************************                  **********

void derezzed()
{

  //      variables locales
  //................................
  //couleur d'affichage
  const uint8_t blanc = 1;
  const uint8_t noir = 0 ;

  //donne la résolution vertical
  const uint8_t resoVertical= TV.vres();

  //fixe l'abcsisse des segments de droite
  const uint8_t x0 = 0;
  const uint8_t x1 = 120;
  const uint8_t horizon = 40;
  const uint8_t ValueHorizonMove  = ((int)avgValue/40);
  const uint8_t ValHorMov  = ((int)avgValue/30);





{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:

  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans: *************** *************** ***************

     TV.draw_line(0,38,118,38,blanc); //Ligne d'horizon blanche
     
//      Réinitialisation de l'affichage:
  TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir pour réinitialiser l'affichage

  TV.draw_line(0,38,120,38,blanc); //Ligne d'horizon blanche

       //      affichage de la perspective façon Grille de Tron
      //.................................
 
     
     TV.draw_line((43+ValueHorizonMove),horizon-1,33,96,blanc);  
     TV.draw_line((57+ValueHorizonMove),horizon-1,87,96,blanc);   
           TV.draw_line(24+ValueHorizonMove,horizon-1,0,96,blanc);  
           TV.draw_line(76+ValueHorizonMove,horizon-1,120,96,blanc);
                TV.draw_line(7+ValueHorizonMove,horizon-1,0,52,blanc);  
                TV.draw_line(93+ValueHorizonMove,horizon-1,120,52,blanc); 
                

           // Montagne derezzed   
         
         TV.draw_line(-6+ValHorMov,horizon-3, 0+ValHorMov,horizon-7,blanc);
                 TV.draw_line(0+ValHorMov,horizon-7, 6+ValHorMov,horizon-3,blanc);

                TV.draw_line(14+ValHorMov,horizon-3, 22+ValHorMov,horizon-10,blanc);
                 TV.draw_line(22+ValHorMov,horizon-10, 28+ValHorMov,horizon-3,blanc);
                 
                  TV.draw_line(22+ValHorMov,horizon-10, 30+ValHorMov,horizon-10,blanc);
                  TV.draw_line(30+ValHorMov,horizon-10, 40+ValHorMov,horizon-17,blanc);
                    TV.draw_line(40+ValHorMov,horizon-17, 46+ValHorMov,horizon-12,blanc);
                   
                     TV.draw_line(46+ValHorMov,horizon-12, 46+ValHorMov,horizon-10,blanc);
                     TV.draw_line(46+ValHorMov,horizon-10, 52+ValHorMov,horizon-3,blanc);
              
                     TV.draw_line(46+ValHorMov,horizon-12, 58+ValHorMov,horizon- 24,blanc);
                            TV.draw_line(58+ValHorMov,horizon-24, 64+ValHorMov,horizon- 10,blanc);
                               TV.draw_line(64+ValHorMov,horizon-10, 76+ValHorMov,horizon- 3,blanc);
                               
                     TV.draw_line(58+ValHorMov,horizon-24, 76+ValHorMov,horizon- 10,blanc);                
                             TV.draw_line(76+ValHorMov,horizon-10, 80+ValHorMov,horizon- 14,blanc); 
                               TV.draw_line(80+ValHorMov,horizon-14, 92+ValHorMov,horizon- 3,blanc); 
    
                   TV.draw_line(100+ValHorMov,horizon-3, 102+ValHorMov,horizon- 5,blanc); 
                   TV.draw_line(102+ValHorMov,horizon-5, 104+ValHorMov,horizon- 3,blanc); 

      //                                  lignes du milieu vers le bas
      //..............................................................................................................

      //      lignes principales
      //.....................................

      //explication du calcul:
      //le calcul en perpective ici utilise un "carré" "x^2" "x*x"
      //cette equation mathematique est assez proche de l'équation d'une perspective!

      uint16_t p =  (  (  ct0 * ct0 ) >> 5 ) +   horizon;
      uint16_t pN = (   (  ( ct0-1) * (ct0-1)  ) >> 5 ) +   horizon;
      uint16_t q = (  (  ct1 * ct1 ) >> 5 ) +   horizon;
      uint16_t qN =(   (  ( ct1-1) * (ct1-1)  ) >> 5 ) +   horizon;
      uint16_t r = (  (  ct2 * ct2 ) >> 5 ) +   horizon;
      uint16_t rN =(   (  ( ct2-1) * (ct2-1)  ) >> 5 ) +   horizon;
      uint16_t s = (  (  ct3 * ct3 ) >> 5 ) +   horizon;
      uint16_t sN =(   (  ( ct3-1) * (ct3-1)  ) >> 5 ) +   horizon;
      uint16_t t = (  (  ct4 * ct4 ) >> 5 ) +   horizon;
      uint16_t tN =(   (  ( ct4-1) * (ct4-1)  ) >> 5 ) +   horizon;
      uint16_t u = (  (  ct5 * ct5 ) >> 5 ) +   horizon;
      uint16_t uN =(   (  ( ct5-1) * (ct5-1)  ) >> 5 ) +   horizon;
      uint16_t v = (  (  ct6 * ct6 ) >> 5 ) +   horizon;
      uint16_t vN =(   (  ( ct6-1) * (ct6-1)  ) >> 5 ) +   horizon;
      uint16_t w = (  (  ct7 * ct7 ) >> 5 ) +   horizon;
      uint16_t wN =(   (  ( ct7-1) * (ct7-1)  ) >> 5 ) +   horizon;

      TV.draw_line(x0,p,x1,p,blanc); 
      TV.draw_line(x0,pN,x1,pN,noir);

      TV.draw_line(x0,q,x1,q,blanc); 
      TV.draw_line(x0,qN,x1,qN,noir);

      TV.draw_line(x0,r,x1,r,blanc); 
      TV.draw_line(x0,rN,x1,rN,noir);

      TV.draw_line(x0,s,x1,s,blanc); 
      TV.draw_line(x0,sN,x1,sN,noir);

      TV.draw_line(x0,t,x1,t,blanc); 
      TV.draw_line(x0,tN,x1,tN,noir);

      TV.draw_line(x0,u,x1,u,blanc); 
      TV.draw_line(x0,uN,x1,uN,noir);

      //      lignes relais de 40 à 168 
      uint16_t a =  (  (  cr0 * cr0 ) >> 5 ) +   horizon;
      uint16_t aN = (   (  ( cr0-1) * (cr0-1)  ) >> 5 ) +   horizon;
      uint16_t b =  (  (  cr1 * cr1 ) >> 5 ) +   horizon;
      uint16_t bN = (   (  ( cr1-1) * (cr1-1)  ) >> 5 ) +   horizon;
      uint16_t c =  (  (  cr2 * cr2 ) >> 5 ) +   horizon;
      uint16_t cN = (   (  ( cr2-1) * (cr2-1)  ) >> 5 ) +   horizon;
      uint16_t d =  (  (  cr3 * cr3 ) >> 5 ) +   horizon;
      uint16_t dN = (   (  ( cr3-1) * (cr3-1)  ) >> 5 ) +   horizon;  
      uint16_t e =  (  (  cr4 * cr4 ) >> 5 ) +   horizon;
      uint16_t eN = (   (  ( cr4-1) * (cr4-1)  ) >> 5 ) +   horizon;  
      uint16_t f =  (  (  cr5 * cr5 ) >> 5 ) +   horizon;
      uint16_t fN = (   (  ( cr5-1) * (cr5-1)  ) >> 5 ) +   horizon;  

      TV.draw_line(x0,a,x1,a,blanc); 
      TV.draw_line(x0,aN,x1,aN,noir);

      TV.draw_line(x0,b,x1,b,blanc); 
      TV.draw_line(x0,bN,x1,bN,noir);

      TV.draw_line(x0,c,x1,c,blanc); 
      TV.draw_line(x0,cN,x1,cN,noir);

      TV.draw_line(x0,d,x1,d,blanc); 
      TV.draw_line(x0,dN,x1,dN,noir);

      TV.draw_line(x0,e,x1,e,blanc); 
      TV.draw_line(x0,eN,x1,eN,noir);

      //                   lecture ADC
      //Pour faire varier un peu l'affichage on implémente une lecture aDC
      lectureADC();

      //         assignation de la lecture aDC au defilement de lignes, valeur mini = 7, valeur max = 10

      int Value= (int)avgValue;

      if (Value <256)
      {
        ecart=10;
      }
      else if (( Value <512) && ( Value>=256) )
      {
        ecart=9;
      }
      else if ( (Value <768) && (Value >=512)  )
      {
        ecart=8;
      }
      else if (Value >=768)
      {
        ecart=7;
      }

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un écran noir pour simuler Gate IN

    }
  }

}
//accolade de samplepin
 }
 
 //                                                        Perspective (Event Horizon)
//..........................................................................................................................................
//******************************************************************************************************************************************


void perspective()
{

  //      variables locales
  //................................
  //couleur d'affichage
  const uint8_t blanc = 1;
  const uint8_t noir = 0 ;

  //donne la résolution vertical
  const uint8_t resoVertical= TV.vres();

  //fixe l'abcsisse des segments de droite
  const uint8_t x0 = 0;
  const uint8_t x1 = 120;
  const uint8_t horizon = 40;


{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:

  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans: *************** *************** ***************



      //      affichage de l'horizon
      //.................................
      TV.draw_line(x0,horizon-1,x1,horizon-1,blanc);  

      //                                  lignes du milieu vers le bas
      //..............................................................................................................

      //      lignes principales
      //.....................................
      TV.select_font(font4x6);

      //explication du calcul:
      //le calcul en perpective ici utilise un "carré" "x^2" "x*x"
      //cette equation mathematique est assez proche de l'équation d'une perpspective!

      uint16_t p =  (  (  ct0 * ct0 ) >> 5 ) +   horizon;
      uint16_t pN = (   (  ( ct0-1) * (ct0-1)  ) >> 5 ) +   horizon;
      uint16_t q = (  (  ct1 * ct1 ) >> 5 ) +   horizon;
      uint16_t qN =(   (  ( ct1-1) * (ct1-1)  ) >> 5 ) +   horizon;
      uint16_t r = (  (  ct2 * ct2 ) >> 5 ) +   horizon;
      uint16_t rN =(   (  ( ct2-1) * (ct2-1)  ) >> 5 ) +   horizon;
      uint16_t s = (  (  ct3 * ct3 ) >> 5 ) +   horizon;
      uint16_t sN =(   (  ( ct3-1) * (ct3-1)  ) >> 5 ) +   horizon;
      uint16_t t = (  (  ct4 * ct4 ) >> 5 ) +   horizon;
      uint16_t tN =(   (  ( ct4-1) * (ct4-1)  ) >> 5 ) +   horizon;
      uint16_t u = (  (  ct5 * ct5 ) >> 5 ) +   horizon;
      uint16_t uN =(   (  ( ct5-1) * (ct5-1)  ) >> 5 ) +   horizon;
      uint16_t v = (  (  ct6 * ct6 ) >> 5 ) +   horizon;
      uint16_t vN =(   (  ( ct6-1) * (ct6-1)  ) >> 5 ) +   horizon;
      uint16_t w = (  (  ct7 * ct7 ) >> 5 ) +   horizon;
      uint16_t wN =(   (  ( ct7-1) * (ct7-1)  ) >> 5 ) +   horizon;

      TV.draw_line(x0,p,x1,p,blanc); 
      TV.draw_line(x0,pN,x1,pN,noir);

      TV.draw_line(x0,q,x1,q,blanc); 
      TV.draw_line(x0,qN,x1,qN,noir);

      TV.draw_line(x0,r,x1,r,blanc); 
      TV.draw_line(x0,rN,x1,rN,noir);

      TV.draw_line(x0,s,x1,s,blanc); 
      TV.draw_line(x0,sN,x1,sN,noir);

      TV.draw_line(x0,t,x1,t,blanc); 
      TV.draw_line(x0,tN,x1,tN,noir);

      TV.draw_line(x0,u,x1,u,blanc); 
      TV.draw_line(x0,uN,x1,uN,noir);

      TV.draw_line(x0,v,x1,v,blanc); 
      TV.draw_line(x0,vN,x1,vN,noir);

      //      lignes relais de 40 à 168 
      uint16_t a =  (  (  cr0 * cr0 ) >> 5 ) +   horizon;
      uint16_t aN = (   (  ( cr0-1) * (cr0-1)  ) >> 5 ) +   horizon;
      uint16_t b =  (  (  cr1 * cr1 ) >> 5 ) +   horizon;
      uint16_t bN = (   (  ( cr1-1) * (cr1-1)  ) >> 5 ) +   horizon;
      uint16_t c =  (  (  cr2 * cr2 ) >> 5 ) +   horizon;
      uint16_t cN = (   (  ( cr2-1) * (cr2-1)  ) >> 5 ) +   horizon;
      uint16_t d =  (  (  cr3 * cr3 ) >> 5 ) +   horizon;
      uint16_t dN = (   (  ( cr3-1) * (cr3-1)  ) >> 5 ) +   horizon;  
      uint16_t e =  (  (  cr4 * cr4 ) >> 5 ) +   horizon;
      uint16_t eN = (   (  ( cr4-1) * (cr4-1)  ) >> 5 ) +   horizon;  
      uint16_t f =  (  (  cr5 * cr5 ) >> 5 ) +   horizon;
      uint16_t fN = (   (  ( cr5-1) * (cr5-1)  ) >> 5 ) +   horizon;  

      TV.draw_line(x0,a,x1,a,blanc); 
      TV.draw_line(x0,aN,x1,aN,noir);

      TV.draw_line(x0,b,x1,b,blanc); 
      TV.draw_line(x0,bN,x1,bN,noir);

      TV.draw_line(x0,c,x1,c,blanc); 
      TV.draw_line(x0,cN,x1,cN,noir);

      TV.draw_line(x0,d,x1,d,blanc); 
      TV.draw_line(x0,dN,x1,dN,noir);

      TV.draw_line(x0,e,x1,e,blanc); 
      TV.draw_line(x0,eN,x1,eN,noir);

      TV.draw_line(x0,f,x1,f,blanc); 
      TV.draw_line(x0,fN,x1,fN,noir);

      //                                  lignes du milieu vers le haut
      //..............................................................................................................

      //      lignes principales
      //.....................................

      //explication du calcul:
      //le calcul en perpective ici utilise un "carré" "x^2" "x*x"
      //cette equation mathematique est assez proche de l'équation d'une perpspective!

      uint16_t g =  horizon  -(  (  ct0 * ct0 ) >> 5 )-2;
      uint16_t gN = horizon - (   (  ( ct0-1) * (ct0-1)  ) >> 5 ) -2 ;
      uint16_t h = horizon - (  (  ct1 * ct1 ) >> 5 ) -2 ;
      uint16_t hN = horizon - (   (  ( ct1-1) * (ct1-1)  ) >> 5 )-2 ;
      uint16_t i = horizon -(  (  ct2 * ct2 ) >> 5 ) -2;
      uint16_t iN = horizon -(   (  ( ct2-1) * (ct2-1)  ) >> 5 )-2 ;
      uint16_t j = horizon -(  (  ct3 * ct3 ) >> 5 )-2;
      uint16_t jN =horizon -(   (  ( ct3-1) * (ct3-1)  ) >> 5 ) -2;
      uint16_t k = horizon -(  (  ct4 * ct4 ) >> 5 ) -2;
      uint16_t kN = horizon -(   (  ( ct4-1) * (ct4-1)  ) >> 5 )-2;
      uint16_t l = horizon -(  (  ct5 * ct5 ) >> 5 ) -2;
      uint16_t lN =horizon -(   (  ( ct5-1) * (ct5-1)  ) >> 5 ) -2;

      TV.draw_line(x0,g,x1,g,blanc); 
      TV.draw_line(x0,gN,x1,gN,noir);

      TV.draw_line(x0,h,x1,h,blanc); 
      TV.draw_line(x0,hN,x1,hN,noir);

      TV.draw_line(x0,i,x1,i,blanc); 
      TV.draw_line(x0,iN,x1,iN,noir);

      TV.draw_line(x0,j,x1,j,blanc); 
      TV.draw_line(x0,jN,x1,jN,noir);

      TV.draw_line(x0,k,x1,k,blanc); 
      TV.draw_line(x0,kN,x1,kN,noir);

      TV.draw_line(x0,l,x1,l,blanc); 
      TV.draw_line(x0,lN,x1,lN,noir);

      //      lignes relais
      //.....................................

      //de 40 à 168 
      uint16_t m = horizon - (  (  cr0 * cr0 ) >> 5 )-2;
      uint16_t mN =horizon - (   (  ( cr0-1) * (cr0-1)  ) >> 5 ) -2;
      uint16_t n = horizon - (  (  cr1 * cr1 ) >> 5 )-2;
      uint16_t nN =horizon -(   (  ( cr1-1) * (cr1-1)  ) >> 5 )-2;
      uint16_t o = horizon - (  (  cr2 * cr2 ) >> 5 )-2;
      uint16_t oN =horizon - (   (  ( cr2-1) * (cr2-1)  ) >> 5 )-2;
      uint16_t x = horizon - (  (  cr3 * cr3 ) >> 5 )-2;
      uint16_t xN =horizon - (   (  ( cr3-1) * (cr3-1)  ) >> 5 ) -2;  
      uint16_t y = horizon - (  (  cr4 * cr4 ) >> 5 ) -2;
      uint16_t yN = horizon -(   (  ( cr4-1) * (cr4-1)  ) >> 5 ) -2;  
      uint16_t z = horizon - (  (  cr5 * cr5 ) >> 5 ) -2;
      uint16_t zN = horizon -(   (  ( cr5-1) * (cr5-1)  ) >> 5 ) -2;  


      TV.draw_line(x0,m,x1,m,blanc); 
      TV.draw_line(x0,mN,x1,mN,noir);

      TV.draw_line(x0,n,x1,n,blanc); 
      TV.draw_line(x0,nN,x1,nN,noir);

      TV.draw_line(x0,o,x1,o,blanc); 
      TV.draw_line(x0,oN,x1,oN,noir);

      TV.draw_line(x0,x,x1,x,blanc); 
      TV.draw_line(x0,xN,x1,xN,noir);

      TV.draw_line(x0,y,x1,y,blanc); 
      TV.draw_line(x0,yN,x1,yN,noir);

      //                   lecture ADC
      //Pour faire varier un peu l'affichage on implémente une lecture aDC
      lectureADC();

      //         assignation de la lecture aDC au defilement de lignes, valeur mini = 7, valeur max = 10

      int Value= (int)avgValue;

      if (Value <256)
      {
        ecart=10;
      }
      else if (( Value <512) && ( Value>=256) )
      {
        ecart=9;
      }
      else if ( (Value <768) && (Value >=512)  )
      {
        ecart=8;
      }
      else if (Value >=768)
      {
        ecart=7;
      }

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un écran noir pour simuler Gate IN

    }
  }

}
//accolade de samplepin
 }


//***************************************************************************************************************************

//                                                           POINT JP 3 marches
//.........................................................................................................................................
//******************************************************************************************************************************************

void PointJP3marches()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;
  uint8_t xB;

  lectureADC(); //Pour faire varier un peu l'affichage on implémente une lecture aDC

  uint8_t Value= ( (  (int)avgValue >> 4 )  );   //lie l'ADC et renvoie en valeur entre 0 et 128
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:

  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:


      //          pattern
      //............................

   //  TV.bitmap(0,0,POINT); //imprime une ligne "POINT"
      //   TV.bitmap(2,3,POINT); //imprime une ligne "POINT"
 
    
   //TV.draw_rect(0, Value+40, 118, Value+40, blanc); 
   
     //draw_rect(x0,y0,width,height,color)
     
     TV.draw_rect(0, Value +60 , 38 ,8 ,1);   //draw_rect(x0,y0,width,height,color)
      TV.draw_rect(0, Value +68, 38 ,8 ,0);   //draw_rect(x0,y0,width,height,color)
 
        TV.draw_rect(40, Value +61 , 38 ,8 ,0);   //draw_rect(x0,y0,width,height,color)
      TV.draw_rect(40, Value +68, 38 ,8 ,1);   //draw_rect(x0,y0,width,height,color)
      
           TV.draw_rect(80, Value +60 , 38 ,8 ,1);   //draw_rect(x0,y0,width,height,color)
      TV.draw_rect(80, Value +68, 38 ,8 ,0);   //draw_rect(x0,y0,width,height,color)
     

          TV.shift(1,0); 

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir au dessus du Viper 

    }
  }

}
//accolade de samplepin
 }

//***************************************************************************************************************************


//                                                           POINT JPQuadrachromic
//.........................................................................................................................................
//******************************************************************************************************************************************

void PointJPQuadrachromic()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;
  uint8_t xB;

  lectureADC(); //Pour faire varier un peu l'affichage on implémente une lecture aDC

  uint8_t Value= ( (  (int)avgValue >> 4 )  );   //lie l'ADC et renvoie en valeur entre 0 et 128
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:


  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:


      //          pattern
      //............................


     //draw_rect(x0,y0,width,height,color)
     
     TV.draw_rect(0, Value +60 , 29 ,8 ,1);   //draw_rect(x0,y0,width,height,color)
      TV.draw_rect(0, Value +68, 29 ,8 ,0);   //draw_rect(x0,y0,width,height,color)
 
        TV.draw_rect(30, Value +61 , 29 ,8 ,0);   //draw_rect(x0,y0,width,height,color)
      TV.draw_rect(30, Value +68, 29 ,8 ,1);   //draw_rect(x0,y0,width,height,color)
      
           TV.draw_rect(60, Value +60 , 29 ,8 ,1);   //draw_rect(x0,y0,width,height,color)
      TV.draw_rect(60, Value +68, 29 ,8 ,0);   //draw_rect(x0,y0,width,height,color)
      
              TV.draw_rect(90, Value +61 , 29 ,8 ,0);   //draw_rect(x0,y0,width,height,color)
      TV.draw_rect(90, Value +68, 29 ,8 ,1);   //draw_rect(x0,y0,width,height,color)


          TV.shift(1,0); 
          


    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir au dessus du Viper 

    }
  }

}

//accolade de samplepin
 }

//***************************************************************************************************************************


//                                                           Brett1
//.........................................................................................................................................
//******************************************************************************************************************************************

void Brett1()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;
  const uint8_t noir = 2;
  uint8_t xB;

  lectureADC(); //Pour faire varier un peu l'affichage on implémente une lecture aDC

  uint8_t Value= ( (  (int)avgValue >> 4 )  );   //lie l'ADC et renvoie en valeur entre 0 et 128
 /*
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 
*/


{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:




  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

      //          pattern
      //............................

{
  //explication:
  //lit l'ADC Value et renvoie en valeur entre 0 et 128

  if (Value < 29 )    
  {
  TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  // TV.invert(); //Invert the data on the screen. 
  }

  
  //******************************** ****************************** ******************************
  
   else if (Value < 30 ) // valeur moyenne, environ 0V selon calibration d'offset du module
  {
   TV.draw_circle(xMilieu,yMilieu +25 ,12,blanc,-1); //draw_circle(x,y,radius,color,fillcolor)
  } 
       else if (Value < 31 )
  {  
   TV.draw_circle(xMilieu,yMilieu +25, 22,blanc,-1); //draw_circle(x,y,radius,color,fillcolor)
  }  
       else if (Value < 32 )
  {  
   TV.draw_circle(xMilieu,yMilieu +25, 32,blanc,-1); //draw_circle(x,y,radius,color,fillcolor)
  } 
       else if (Value < 33 )
  {
   TV.draw_circle(xMilieu,yMilieu +25, 42,blanc,-1); //draw_circle(x,y,radius,color,fillcolor)
  }  
       else if (Value < 34 )
  { 
   TV.draw_rect(xMilieu-6, yMilieu+19, 12, 12, blanc, -1); //draw_rect(x0,y0,width,height,color,fillcolor)
  } 
          else if (Value < 35 )
  { 
   TV.draw_rect(xMilieu-11, yMilieu+14, 22, 22, blanc, -1); //draw_rect(x0,y0,width,height,color,fillcolor)
  } 
          else if (Value < 36 )
  {    
   TV.draw_rect(xMilieu-22, yMilieu+3, 44, 44, blanc, -1); //draw_rect(x0,y0,width,height,color,fillcolor)
  }  
          else if (Value < 37 )
  {
   TV.draw_rect(xMilieu-32, yMilieu-7, 64, 64, blanc, -1); //draw_rect(x0,y0,width,height,color,fillcolor)
  } 
          else if (Value < 38 )
  {
   TV.draw_rect(xMilieu-42, yMilieu-17, 84, 84, blanc, -1); //draw_rect(x0,y0,width,height,color,fillcolor)
  }   
  
            else if (Value < 40 )
  { 
   TV.draw_rect(xMilieu-42, yMilieu+19, 84, 12, blanc, 1); //Croix part 1
  }   
            else if (Value < 41 )
  {
    
   TV.draw_rect(xMilieu-6, yMilieu-17, 12, 84, blanc, 1); //Croix part 2
  }     
  
            else if (Value < 42 )
  {
    TV.draw_rect(xMilieu+1, yMilieu+2, 22, 22, blanc, 1); // carré damier
    TV.draw_rect(xMilieu-22, yMilieu+25, 22, 22, blanc, 1); // carré damier
  } 
  
             else if (Value < 43 )
  {
    TV.draw_rect(xMilieu-22, yMilieu+2, 22, 22, blanc, 1); // carré damier
    TV.draw_rect(xMilieu+1, yMilieu+25, 22, 22, blanc, 1); // carré damier
  }  
  
            else if (Value < 44 )
  {
   TV.draw_rect(xMilieu-6, yMilieu+19, 12, 12, blanc, 1); //Un petit carré rempli blanc
  }  

           else if (Value < 45 )
  {
   TV.draw_rect(xMilieu-54, yMilieu-12, 12, 2, blanc, 1); //Une ligne horizontale de rectangle éspacés pour former des croix -Ligne1
     TV.draw_rect(xMilieu-30, yMilieu-12, 12, 2, blanc, 1); 
       TV.draw_rect(xMilieu-6, yMilieu-12, 12, 2, blanc, 1); 
         TV.draw_rect(xMilieu+18, yMilieu-12, 12, 2, blanc, 1); 
           TV.draw_rect(xMilieu+42, yMilieu-12, 12, 2, blanc, 1);  
  }  

           else if (Value < 46 )
  {
    
   TV.draw_rect(xMilieu-54, yMilieu+12, 12, 2, blanc, 1); //Une ligne horizontale de rectangle éspacés pour former des croix -Ligne2
     TV.draw_rect(xMilieu-30, yMilieu+12, 12, 2, blanc, 1); 
       TV.draw_rect(xMilieu-6, yMilieu+12, 12, 2, blanc, 1); 
         TV.draw_rect(xMilieu+18, yMilieu+12, 12, 2, blanc, 1); 
           TV.draw_rect(xMilieu+42, yMilieu+12, 12, 2, blanc, 1);  
  }  
  
           else if (Value < 47 )
  {
    
   TV.draw_rect(xMilieu-54, yMilieu+36, 12, 2, blanc, 1); //Une ligne horizontale de rectangle éspacés pour former des croix -Ligne3
     TV.draw_rect(xMilieu-30, yMilieu+36, 12, 2, blanc, 1); 
       TV.draw_rect(xMilieu-6, yMilieu+36, 12, 2, blanc, 1); 
         TV.draw_rect(xMilieu+18, yMilieu+36, 12, 2, blanc, 1); 
           TV.draw_rect(xMilieu+42, yMilieu+36, 12, 2, blanc, 1);  
  }  

           else if (Value < 48 )
  {
    
   TV.draw_rect(xMilieu-54, yMilieu+60, 12, 2, blanc, 1); //Une ligne horizontale de rectangle éspacés pour former des croix -Ligne4
     TV.draw_rect(xMilieu-30, yMilieu+60, 12, 2, blanc, 1); 
       TV.draw_rect(xMilieu-6, yMilieu+60, 12, 2, blanc, 1); 
         TV.draw_rect(xMilieu+18, yMilieu+60, 12, 2, blanc, 1); 
           TV.draw_rect(xMilieu+42, yMilieu+60, 12, 2, blanc, 1);  
  } 
           else if (Value < 50 )
  {
    
   TV.draw_rect(xMilieu-49, yMilieu-17, 2, 12, blanc, 1); //Une ligne verticale de rectangle éspacés pour former des croix Ligne1
    TV.draw_rect(xMilieu-25, yMilieu-17, 2, 12, blanc, 1);
      TV.draw_rect(xMilieu-1, yMilieu-17, 2, 12, blanc, 1);
         TV.draw_rect(xMilieu+23, yMilieu-17, 2, 12, blanc, 1);    
           TV.draw_rect(xMilieu+47, yMilieu-17, 2, 12, blanc, 1);  
  }
           else if (Value < 51 )
  {
   TV.draw_rect(xMilieu-49, yMilieu+7, 2, 12, blanc, 1); //Une ligne verticale de rectangle éspacés pour former des croix Ligne2
    TV.draw_rect(xMilieu-25, yMilieu+7, 2, 12, blanc, 1);
      TV.draw_rect(xMilieu-1, yMilieu+7, 2, 12, blanc, 1);
         TV.draw_rect(xMilieu+23, yMilieu+7, 2, 12, blanc, 1);    
           TV.draw_rect(xMilieu+47, yMilieu+7, 2, 12, blanc, 1);  
  }
           else if (Value < 52 )
  {
   TV.draw_rect(xMilieu-49, yMilieu+31, 2, 12, blanc, 1); //Une ligne verticale de rectangle éspacés pour former des croix Ligne2
    TV.draw_rect(xMilieu-25, yMilieu+31, 2, 12, blanc, 1);
      TV.draw_rect(xMilieu-1, yMilieu+31, 2, 12, blanc, 1);
         TV.draw_rect(xMilieu+23, yMilieu+31, 2, 12, blanc, 1);    
           TV.draw_rect(xMilieu+47, yMilieu+31, 2, 12, blanc, 1);  
  }
           else if (Value < 53 )
  {
    
   TV.draw_rect(xMilieu-49, yMilieu+55, 2, 12, blanc, 1); //Une ligne verticale de rectangle éspacés pour former des croix Ligne2
    TV.draw_rect(xMilieu-25, yMilieu+55, 2, 12, blanc, 1);
      TV.draw_rect(xMilieu-1, yMilieu+55, 2, 12, blanc, 1);
         TV.draw_rect(xMilieu+23, yMilieu+55, 2, 12, blanc, 1);    
           TV.draw_rect(xMilieu+47, yMilieu+55, 2, 12, blanc, 1);  
  }

 // draw_line(x0,y0,x1,y1,color)
  
           else if (Value < 54 )
  {
    TV.draw_line(xMilieu,yMilieu-19,xMilieu-49,yMilieu+66,1); //Une ligne pour figurer une tangeante du triangle coté G
     TV.draw_line(xMilieu,yMilieu-19,xMilieu+49,yMilieu+66,1); //Une ligne pour figurer une tangeante du triangle coté Dt
      TV.draw_line(xMilieu-49,yMilieu+66,xMilieu+49,yMilieu+66,1); //Une ligne pour figurer une tangeante du triangle en bas
  }
            else if (Value < 55 )
  {
    TV.draw_line(xMilieu,yMilieu-5,xMilieu-34,yMilieu+54,1); //Une ligne pour figurer une tangeante du triangle coté G
     TV.draw_line(xMilieu,yMilieu-5,xMilieu+34,yMilieu+54,1); //Une ligne pour figurer une tangeante du triangle coté Dt
      TV.draw_line(xMilieu-34,yMilieu+54,xMilieu+34,yMilieu+54,1); //Une ligne pour figurer une tangeante du triangle en bas
  } 
  
              else if (Value < 56 )
  {
    TV.draw_line(xMilieu,yMilieu+9,xMilieu-20,yMilieu+44,1); //Une ligne pour figurer une tangeante du triangle coté G
     TV.draw_line(xMilieu,yMilieu+9,xMilieu+20,yMilieu+44,1); //Une ligne pour figurer une tangeante du triangle coté Dt
      TV.draw_line(xMilieu-20,yMilieu+44,xMilieu+20,yMilieu+44,1); //Une ligne pour figurer une tangeante du triangle en bas
  } 

            else if (Value < 57 )
  {
  // TV.bitmap(16,38,LOGODUJAP);
  }  
  
            else if (Value < 58 )
  {
    
    TV.draw_circle(xMilieu,yMilieu +25, 42,blanc,1); //Un gros rond blanc
  }   


 /*           else if (Value < 58 )
  {
    
    TV.draw_rect(0, 0, 118, 96, blanc, 1); //Un gros carré blanc
  }  
*/

       else if (Value < 120 )
  {
      TV.invert(); //Invert the data on the screen. 
  } 
 
}
    } 

    else {    // et si condition de trig Input Low n'est pas remplie:

    TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  
    }
  }

}
//accolade de samplepin
 }

//                                                           Gros Pixels
//.........................................................................................................................................
//******************************************************************************************************************************************

void GrosPixels()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;
  const uint8_t noir = 2;
  uint8_t xB;

  lectureADC(); //Pour faire varier un peu l'affichage on implémente une lecture aDC

  uint8_t Value= ( (  (int)avgValue >> 4 )  );   //lie l'ADC et renvoie en valeur entre 0 et 128
 /*
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 
*/


{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:

  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

      //          pattern
      //............................

{
  //explication:
  //lit l'ADC Value et renvoie en valeur entre 0 et 128

  if (Value < 29 )    
  {
  TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  // TV.invert(); //Invert the data on the screen. 
  }

  
  //******************************** ****************************** ******************************
  
     else if (Value < 30 ) // valeur moyenne, environ 0V selon calibration d'offset du module
  {
    TV.draw_rect(xMilieu-60, yMilieu-14, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
    
  } 
       else if (Value < 31 )
  {  
    TV.draw_rect(xMilieu-40, yMilieu-14, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
  }  
       else if (Value < 32 )
  {  
    TV.draw_rect(xMilieu-20, yMilieu-14, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
  } 
       else if (Value < 33 )
  {
    TV.draw_rect(xMilieu, yMilieu-14, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
  }  
       else if (Value < 34 )
  { 
    TV.draw_rect(xMilieu +20, yMilieu-14, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
  } 
          else if (Value < 35 )
  { 
    TV.draw_rect(xMilieu +40, yMilieu-14, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
  } 

          else if (Value < 36 )
  {
    TV.draw_rect(xMilieu-60, yMilieu+6, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite Nouvelle Ligne
  } 
          else if (Value < 37 )
  {
    TV.draw_rect(xMilieu-40, yMilieu+6, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
  }   
  
            else if (Value < 38 )
  { 
    TV.draw_rect(xMilieu-20, yMilieu+6, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
  }   
            else if (Value < 39 )
  { 
    TV.draw_rect(xMilieu, yMilieu+6, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
  }     
  
            else if (Value < 40 )
  {
    TV.draw_rect(xMilieu+20, yMilieu+6, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
  } 
  
             else if (Value < 41 )
  {
    TV.draw_rect(xMilieu+40, yMilieu+6, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
  }  


           else if (Value < 42 )
  {
    TV.draw_rect(xMilieu-60, yMilieu+26, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite Nouvelle Ligne
  }  

           else if (Value < 43 )
  {
    TV.draw_rect(xMilieu-40, yMilieu+26, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite   
 
  }  
  
           else if (Value < 44 )
  {
     TV.draw_rect(xMilieu-20, yMilieu+26, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite      
  
  }  

           else if (Value < 45 )
  {
     TV.draw_rect(xMilieu, yMilieu+26, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite        

  } 
           else if (Value < 46 )
  {
     TV.draw_rect(xMilieu+20, yMilieu+26, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite        

  }
           else if (Value < 47 )
  {
      TV.draw_rect(xMilieu+40, yMilieu+26, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite      
  }
           else if (Value < 48 )

  {
     TV.draw_rect(xMilieu-60, yMilieu+46, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite Nouvelle Ligne   

  }

           else if (Value < 49 )
  {
     TV.draw_rect(xMilieu-40, yMilieu+46, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite
  }
            else if (Value < 50 )
  {
     TV.draw_rect(xMilieu-20, yMilieu+46, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite
  } 
  
              else if (Value < 51 )
  {
      TV.draw_rect(xMilieu, yMilieu+46, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite 
    
  } 

            else if (Value < 52 )
  {
     TV.draw_rect(xMilieu+20, yMilieu+46, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite
  }  
  
            else if (Value < 53 )
  {
     TV.draw_rect(xMilieu+40, yMilieu+46, 18, 18, blanc, 1); //Un carré Blanc, de gauche à droite   

  }   


       else if (Value < 120 )
  {
      TV.invert(); //Invert the data on the screen. 
  } 
 
}
    } 

    else {    // et si condition de trig Input Low n'est pas remplie:

    TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  
    }
  }

}
//accolade de samplepin
 }
 
 
 
//***************************************************************************************************************************

//                                                           VIPER MARK II
//.........................................................................................................................................
//******************************************************************************************************************************************

void vipersMarkII()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;

  uint8_t y;
  uint8_t yB;


  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:


  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

//++++++++++++++++++++


  //          pattern
  //............................
  //set_pixel(x,y,color)
  y= random ( vRes) ;
  TV.set_pixel (120, y, blanc);
  TV.set_pixel (120-1, y, blanc);

  yB= random (vRes) ;                        //Imprime des ray de vitesse lumière
  TV.draw_line(0,yB,116-y,yB,blanc); 

  TV.bitmap(xMilieu+5,Value+10,VIPER); //imprime un Vaisseau Viper
  TV.bitmap(xMilieu+4,Value+6,VIPERBLACK); //imprime un cache noir au dessus du Viper 
  TV.bitmap(xMilieu+6,Value+23,VIPERBLACK); //imprime un cache noir en dessous du Viper 

  //mouvement de l'écran
  //shift(distance,direction)   direction =>  0=up 1 = down 2 = left 3=right 
  TV.shift(2,2) ; 

//+++++++++++++++++++++

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir au dessus du Viper 

    }
  }

}

//accolade de samplepin
 }
 
//                                                           Cpt Blood
//.........................................................................................................................................
//******************************************************************************************************************************************

void CptBlood()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;

  uint8_t y;
  uint8_t yB;





  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 

//_____________

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:


  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

//++++++++++++++++++++


  //          pattern
  //............................

   TV.bitmap(xMilieu-15,Value,CPTBLOODBABY); //imprime un BB Cpt Blood
  
    
  //mouvement de l'écran
  //shift(distance,direction)   direction =>  0=up 1 = down 2 = left 3=right 
  TV.shift(1,2) ; 

//+++++++++++++++++++++

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir au dessus du Viper 

    }
  }

}

//accolade de samplepin
 }
 
//                                                           Mazinger
//.........................................................................................................................................
//******************************************************************************************************************************************

void Mazinger()

{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;  
  const uint8_t yMilieu = vRes >> 2 ;  
  const uint8_t blanc = 1;

  uint8_t xB;

  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();
  


  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 4 )  );
  if (Value >= ( vRes - 5) )
  {
    Value = ( vRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 

//____________

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:


  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

//++++++++++++++++++++

//          pattern
  //............................

 TV.bitmap(42,0,MAZINGER); //imprime un Mazinger Z
 xB= random ( hRes) ;
 TV.draw_line(Value, xB, Value, 62,blanc); 
 
  TV.shift(1,0); 



//+++++++++++++++++++++

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir au dessus du Viper 

    }
  }

}

//accolade de samplepin
 }
 
//                                                                 WARP
//.........................................................................................................................................
//******************************************************************************************************************************************
void warp()
{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t xMilieu = hRes >> 1 ;
  const uint8_t yMilieu = vRes >> 1 ;
  const uint8_t blanc = 1;
  const uint8_t noir =0;
  uint8_t x;
  uint8_t y;
  uint8_t tempsDaffichage;
  static uint8_t premiereFois = 1;

  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 16
  uint8_t Value= ( (  (int)avgValue >> 6 )  );
  tempsDaffichage = Value;
  if (tempsDaffichage < 5)
  {
    tempsDaffichage = 2;
  }


//____________

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:


  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

//++++++++++++++++++++
  //          pattern
  //............................
  x= random ( hRes) ;
  y= random ( vRes );

/*

  if (1 == premiereFois )
  {
    //draw_rect(x0,y0,width,height,color,fillcolor)
    //draw_circle(x,y,radius,color,fillcolor) 
    //ATTENTION a ne pas dépasser un radius de plus de 50
    TV.clear_screen();
    for (uint16_t i = 0 ; i<16317; i++)
    {
      x= random ( hRes) ;
      y= random ( vRes );
      TV.set_pixel (x, y, blanc);
    }
    TV.draw_rect(0,0,hRes,vRes,blanc,blanc);
    for (uint8_t i = 0 ; i<200; i++)
    {
      TV.draw_circle(xMilieu,yMilieu,i>>2,noir,noir);

    }
    
    
    //ré initialisation de la variable
    premiereFois = 0;
  }


*/



  if (compteurISRglobal<tempsDaffichage)
  {   
    TV.draw_line(xMilieu,yMilieu,x,y,2);
    
    

    //TV.draw_line(xMilieu,yMilieu,x,y,noir);

  }
  else
  {
    //desaffichage
    //draw_circle(x,y,radius,color,fillcolor)    ATTENTION a ne pas dépasser un radius de plus de 50

    for (uint8_t i = 0 ; i<14; i++)
    {
     TV.draw_circle(xMilieu,yMilieu,i<<2,0,0);

    }
  }

  //reinitialisation de la variable
  //................................
  //quand on atteint deux fois le temps d'affichage on reinitialise
  if (compteurISRglobal >=  (tempsDaffichage<<1) )
  {
    compteurISRglobal = 0;
  }

//+++++++++++++++++++++

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir au dessus du Viper 

    }
  }

}

//accolade de samplepin
 }

//                                                                SNAKE 
//.........................................................................................................................................
//******************************************************************************************************************************************

void snake()
{
  //    variables locales
  //......................
  const uint8_t hRes = TV.hres();
  const uint8_t vRes= TV.vres();
  const uint8_t yMilieu = vRes >> 2 ;
  const uint8_t blanc = 1;
  const uint8_t noir =0;
  const uint8_t radius = 2;
  const uint8_t tempsDaffichage = 1;
  uint8_t x;
  const uint8_t y = 1;



  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();



  //                 assignation de la lecture aDC 
  //..................................................................................
  //lie l'ADC et renvoie en valeur entre 0 et 128
  uint8_t Value= ( (  (int)avgValue >> 3 )  );
  if (Value >= ( hRes -5) )
  {
    Value = ( hRes - 5 );
  }
  if (Value <= 3)
  {
    Value = 3;
  } 

//____________

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
    TV.invert(); //Invert the data on the screen. ;
    // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:


  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

//++++++++++++++++++++

  //          pattern
  //............................

  //draw_circle(x,y,radius,color,fillcolor)
  TV.draw_circle(Value,yMilieu,radius,blanc,blanc);

  //set_pixel(x,y,color)
  x= random ( hRes) ;
  TV.set_pixel (x, y, blanc);
  TV.set_pixel (x, y+1, blanc);
  
  //mouvement de l'écran
  //shift(distance,direction)   direction =>  0=up 1 = down 2 = left 3=right 
  TV.shift(1,1) ; 

//+++++++++++++++++++++

    } 
    else {    // et si condition de trig Input Low n'est pas remplie:

      TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir au dessus du Viper 

    }
  }

}

//accolade de samplepin
 }
 

 

 //                                                           10 point algorythme
//.........................................................................................................................................
//******************************************************************************************************************************************

void tenPointAlgorythme()
{
    
  //                   lecture ADC
  //........................................................
  //Pour faire varier un peu l'affichage on implémente une lecture aDC
  lectureADC();

{
if(digitalRead(samplePin) == HIGH) // Utilise le bouton de samplePin de l'oscilloscope
    {
      int i=i+1;
   // TV.invert(); //Invert the data on the screen. ;
     //TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir 
        TV.delay(i);
     
  }
    else     // et si condition de samplePin high n'est pas remplie on passe au pattern de Brett:


  {
    blackScreen = digitalRead(trigIn);

    if (blackScreen == HIGH) {   // Mettre le pattern la dedans:

//++++++++++++++++++++

  //          pattern
/*............................
 *  Based on C64's 10 PRINT CHR$(205.5+RND(1)); : GOTO 10
 *  See: https://10print.org
 *  Ported to TVout by Danny Walker, 2018
 *  http://danny.makesthings.work
 */

int gridSize = 8; // The size of the tiles. Feel free to play with this and see what happens! It's best to keep it to a common factor of 128 and 96, so 4, 8, 16 etc.

  //TV.clear_screen(); // Wipe the screen clean so we can draw a new set of tiles to it.
        TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir
  
  for(int y=0;y<TV.vres();y+=gridSize){ // Loop over the horizontal rows of the tile grid.
    for(int x=0;x<TV.hres();x+= gridSize){ // For each row, loop over each individual tile horizontally.
      
     
      int randomNum = random(0,1024); // Generate a random number from 0-1023
      
      if(randomNum > avgValue){
        // Draw a forward slash
        TV.draw_line(x, y+gridSize, x+gridSize, y,1);
     
      } else {
        // Draw a backward slash
        TV.draw_line(x,y,x+gridSize, y+gridSize,1);
      }
    }

  }

}
    else {    // et si condition de trig Input Low n'est pas remplie:

     // TV.bitmap(0,0,BLACKSCREEN); //imprime un cache noir
      int iGate=iGate+1; // incremente le delay tant que l'impulsion de Gate est négative
      TV.delay(iGate);
    }
  }

}

//accolade de samplepin
 }


 
